@@ -6,6 +6,8 @@
 
 #include "Link.h"
 
+#include "mozilla/MemoryReporting.h"
+#include "mozilla/dom/Element.h"
 #include "nsEventStates.h"
 #include "nsIURL.h"
 #include "nsISizeOf.h"
@@ -23,7 +25,8 @@ namespace dom {
 Link::Link(Element *aElement)
   : mElement(aElement)
   , mHistory(services::GetHistoryService())
-  , mLinkState(defaultState)
+  , mLinkState(eLinkState_NotLink)
+  , mNeedsRegistration(false)
   , mRegistered(false)
 {
   NS_ABORT_IF_FALSE(mElement, "Must have an element");
@@ -34,14 +37,11 @@ Link::~Link()
   UnregisterFromHistory();
 }
 
-nsLinkState
-Link::GetLinkState() const
+bool
+Link::ElementHasHref() const
 {
-  NS_ASSERTION(mRegistered,
-               "Getting the link state of an unregistered Link!");
-  NS_ASSERTION(mLinkState != eLinkState_Unknown,
-               "Getting the link state with an unknown value!");
-  return nsLinkState(mLinkState);
+  return ((!mElement->IsSVG() && mElement->HasAttr(kNameSpaceID_None, nsGkAtoms::href))
+        || (!mElement->IsHTML() && mElement->HasAttr(kNameSpaceID_XLink, nsGkAtoms::href)));
 }
 
 void
@@ -73,36 +73,28 @@ Link::LinkState() const
   // track that state.  Cast away that constness!
   Link *self = const_cast<Link *>(this);
 
-  // If we are not in the document, default to not visited.
   Element *element = self->mElement;
-  if (!element->IsInDoc()) {
-    self->mLinkState = eLinkState_Unvisited;
-  }
 
-  // If we have not yet registered for notifications and are in an unknown
-  // state, register now!
-  if (!mRegistered && mLinkState == eLinkState_Unknown) {
-    // First, make sure the href attribute has a valid link (bug 23209).
+  // If we have not yet registered for notifications and need to,
+  // due to our href changing, register now!
+  if (!mRegistered && mNeedsRegistration && element->IsInDoc()) {
+    // Only try and register once.
+    self->mNeedsRegistration = false;
+
     nsCOMPtr<nsIURI> hrefURI(GetURI());
-    if (!hrefURI) {
-      self->mLinkState = eLinkState_NotLink;
-      return nsEventStates();
-    }
 
     // Assume that we are not visited until we are told otherwise.
     self->mLinkState = eLinkState_Unvisited;
 
-    // We have a good href, so register with History.
-    if (mHistory) {
+    // Make sure the href attribute has a valid link (bug 23209).
+    // If we have a good href, register with History if available.
+    if (mHistory && hrefURI) {
       nsresult rv = mHistory->RegisterVisitedCallback(hrefURI, self);
       if (NS_SUCCEEDED(rv)) {
         self->mRegistered = true;
 
         // And make sure we are in the document's link map.
-        nsIDocument *doc = element->GetCurrentDoc();
-        if (doc) {
-          doc->AddStyleRelevantLink(self);
-        }
+        element->GetCurrentDoc()->AddStyleRelevantLink(self);
       }
     }
   }
@@ -119,36 +111,29 @@ Link::LinkState() const
   return nsEventStates();
 }
 
-already_AddRefed<nsIURI>
+nsIURI*
 Link::GetURI() const
 {
-  nsCOMPtr<nsIURI> uri(mCachedURI);
-
   // If we have this URI cached, use it.
-  if (uri) {
-    return uri.forget();
+  if (mCachedURI) {
+    return mCachedURI;
   }
 
   // Otherwise obtain it.
   Link *self = const_cast<Link *>(this);
   Element *element = self->mElement;
-  uri = element->GetHrefURI();
+  mCachedURI = element->GetHrefURI();
 
-  // We want to cache the URI if the node is in the document.
-  if (uri && element->IsInDoc()) {
-    mCachedURI = uri;
-  }
-
-  return uri.forget();
+  return mCachedURI;
 }
 
-nsresult
+void
 Link::SetProtocol(const nsAString &aProtocol)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   if (!uri) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
   nsAString::const_iterator start, end;
@@ -159,128 +144,152 @@ Link::SetProtocol(const nsAString &aProtocol)
   (void)uri->SetScheme(NS_ConvertUTF16toUTF8(Substring(start, iter)));
 
   SetHrefAttribute(uri);
-  return NS_OK;
 }
 
-nsresult
+void
+Link::SetPassword(const nsAString &aPassword)
+{
+  nsCOMPtr<nsIURI> uri(GetURIToMutate());
+  if (!uri) {
+    // Ignore failures to be compatible with NS4.
+    return;
+  }
+
+  uri->SetPassword(NS_ConvertUTF16toUTF8(aPassword));
+  SetHrefAttribute(uri);
+}
+
+void
+Link::SetUsername(const nsAString &aUsername)
+{
+  nsCOMPtr<nsIURI> uri(GetURIToMutate());
+  if (!uri) {
+    // Ignore failures to be compatible with NS4.
+    return;
+  }
+
+  uri->SetUsername(NS_ConvertUTF16toUTF8(aUsername));
+  SetHrefAttribute(uri);
+}
+
+void
 Link::SetHost(const nsAString &aHost)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   if (!uri) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
-  // We cannot simply call nsIURI::SetHost because that would treat the name as
-  // an IPv6 address (like http:://[server:443]/).  We also cannot call
-  // nsIURI::SetHostPort because that isn't implemented.  Sadfaces.
-
-  // First set the hostname.
-  nsAString::const_iterator start, end;
-  aHost.BeginReading(start);
-  aHost.EndReading(end);
-  nsAString::const_iterator iter(start);
-  (void)FindCharInReadable(':', iter, end);
-  NS_ConvertUTF16toUTF8 host(Substring(start, iter));
-  (void)uri->SetHost(host);
-
-  // Also set the port if needed.
-  if (iter != end) {
-    iter++;
-    if (iter != end) {
-      nsAutoString portStr(Substring(iter, end));
-      nsresult rv;
-      int32_t port = portStr.ToInteger(&rv);
-      if (NS_SUCCEEDED(rv)) {
-        (void)uri->SetPort(port);
-      }
-    }
-  };
-
+  (void)uri->SetHostPort(NS_ConvertUTF16toUTF8(aHost));
   SetHrefAttribute(uri);
-  return NS_OK;
+  return;
 }
 
-nsresult
+void
 Link::SetHostname(const nsAString &aHostname)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   if (!uri) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
   (void)uri->SetHost(NS_ConvertUTF16toUTF8(aHostname));
   SetHrefAttribute(uri);
-  return NS_OK;
 }
 
-nsresult
+void
 Link::SetPathname(const nsAString &aPathname)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
   if (!url) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
   (void)url->SetFilePath(NS_ConvertUTF16toUTF8(aPathname));
   SetHrefAttribute(uri);
-  return NS_OK;
 }
 
-nsresult
-Link::SetSearch(const nsAString &aSearch)
+void
+Link::SetSearch(const nsAString& aSearch)
+{
+  SetSearchInternal(aSearch);
+  if (mSearchParams) {
+    mSearchParams->Invalidate();
+  }
+}
+
+void
+Link::SetSearchInternal(const nsAString& aSearch)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
   if (!url) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
   (void)url->SetQuery(NS_ConvertUTF16toUTF8(aSearch));
   SetHrefAttribute(uri);
-  return NS_OK;
 }
 
-nsresult
+void
 Link::SetPort(const nsAString &aPort)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   if (!uri) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
   nsresult rv;
   nsAutoString portStr(aPort);
-  int32_t port = portStr.ToInteger(&rv);
-  if (NS_FAILED(rv)) {
-    return NS_OK;
+
+  // nsIURI uses -1 as default value.
+  int32_t port = -1;
+  if (!aPort.IsEmpty()) {
+    port = portStr.ToInteger(&rv);
+    if (NS_FAILED(rv)) {
+      return;
+    }
   }
 
   (void)uri->SetPort(port);
   SetHrefAttribute(uri);
-  return NS_OK;
 }
 
-nsresult
+void
 Link::SetHash(const nsAString &aHash)
 {
   nsCOMPtr<nsIURI> uri(GetURIToMutate());
   if (!uri) {
     // Ignore failures to be compatible with NS4.
-    return NS_OK;
+    return;
   }
 
   (void)uri->SetRef(NS_ConvertUTF16toUTF8(aHash));
   SetHrefAttribute(uri);
-  return NS_OK;
 }
 
-nsresult
+void
+Link::GetOrigin(nsAString &aOrigin)
+{
+  aOrigin.Truncate();
+
+  nsCOMPtr<nsIURI> uri(GetURI());
+  if (!uri) {
+    return;
+  }
+
+  nsString origin;
+  nsContentUtils::GetUTFNonNullOrigin(uri, origin);
+  aOrigin.Assign(origin);
+}
+
+void
 Link::GetProtocol(nsAString &_protocol)
 {
   nsCOMPtr<nsIURI> uri(GetURI());
@@ -292,11 +301,41 @@ Link::GetProtocol(nsAString &_protocol)
     (void)uri->GetScheme(scheme);
     CopyASCIItoUTF16(scheme, _protocol);
   }
-  _protocol.Append(PRUnichar(':'));
-  return NS_OK;
+  _protocol.Append(char16_t(':'));
+  return;
 }
 
-nsresult
+void
+Link::GetUsername(nsAString& aUsername)
+{
+  aUsername.Truncate();
+
+  nsCOMPtr<nsIURI> uri(GetURI());
+  if (!uri) {
+    return;
+  }
+
+  nsAutoCString username;
+  uri->GetUsername(username);
+  CopyASCIItoUTF16(username, aUsername);
+}
+
+void
+Link::GetPassword(nsAString &aPassword)
+{
+  aPassword.Truncate();
+
+  nsCOMPtr<nsIURI> uri(GetURI());
+  if (!uri) {
+    return;
+  }
+
+  nsAutoCString password;
+  uri->GetPassword(password);
+  CopyASCIItoUTF16(password, aPassword);
+}
+
+void
 Link::GetHost(nsAString &_host)
 {
   _host.Truncate();
@@ -304,7 +343,7 @@ Link::GetHost(nsAString &_host)
   nsCOMPtr<nsIURI> uri(GetURI());
   if (!uri) {
     // Do not throw!  Not having a valid URI should result in an empty string.
-    return NS_OK;
+    return;
   }
 
   nsAutoCString hostport;
@@ -312,10 +351,9 @@ Link::GetHost(nsAString &_host)
   if (NS_SUCCEEDED(rv)) {
     CopyUTF8toUTF16(hostport, _host);
   }
-  return NS_OK;
 }
 
-nsresult
+void
 Link::GetHostname(nsAString &_hostname)
 {
   _hostname.Truncate();
@@ -323,7 +361,7 @@ Link::GetHostname(nsAString &_hostname)
   nsCOMPtr<nsIURI> uri(GetURI());
   if (!uri) {
     // Do not throw!  Not having a valid URI should result in an empty string.
-    return NS_OK;
+    return;
   }
 
   nsAutoCString host;
@@ -333,10 +371,9 @@ Link::GetHostname(nsAString &_hostname)
   if (NS_SUCCEEDED(rv)) {
     CopyUTF8toUTF16(host, _hostname);
   }
-  return NS_OK;
 }
 
-nsresult
+void
 Link::GetPathname(nsAString &_pathname)
 {
   _pathname.Truncate();
@@ -346,17 +383,17 @@ Link::GetPathname(nsAString &_pathname)
   if (!url) {
     // Do not throw!  Not having a valid URI or URL should result in an empty
     // string.
-    return NS_OK;
+    return;
   }
 
   nsAutoCString file;
   nsresult rv = url->GetFilePath(file);
-  NS_ENSURE_SUCCESS(rv, rv);
-  CopyUTF8toUTF16(file, _pathname);
-  return NS_OK;
+  if (NS_SUCCEEDED(rv)) {
+    CopyUTF8toUTF16(file, _pathname);
+  }
 }
 
-nsresult
+void
 Link::GetSearch(nsAString &_search)
 {
   _search.Truncate();
@@ -366,7 +403,7 @@ Link::GetSearch(nsAString &_search)
   if (!url) {
     // Do not throw!  Not having a valid URI or URL should result in an empty
     // string.
-    return NS_OK;
+    return;
   }
 
   nsAutoCString search;
@@ -374,10 +411,9 @@ Link::GetSearch(nsAString &_search)
   if (NS_SUCCEEDED(rv) && !search.IsEmpty()) {
     CopyUTF8toUTF16(NS_LITERAL_CSTRING("?") + search, _search);
   }
-  return NS_OK;
 }
 
-nsresult
+void
 Link::GetPort(nsAString &_port)
 {
   _port.Truncate();
@@ -385,7 +421,7 @@ Link::GetPort(nsAString &_port)
   nsCOMPtr<nsIURI> uri(GetURI());
   if (!uri) {
     // Do not throw!  Not having a valid URI should result in an empty string.
-    return NS_OK;
+    return;
   }
 
   int32_t port;
@@ -397,10 +433,9 @@ Link::GetPort(nsAString &_port)
     portStr.AppendInt(port, 10);
     _port.Assign(portStr);
   }
-  return NS_OK;
 }
 
-nsresult
+void
 Link::GetHash(nsAString &_hash)
 {
   _hash.Truncate();
@@ -409,53 +444,72 @@ Link::GetHash(nsAString &_hash)
   if (!uri) {
     // Do not throw!  Not having a valid URI should result in an empty
     // string.
-    return NS_OK;
+    return;
   }
 
   nsAutoCString ref;
   nsresult rv = uri->GetRef(ref);
   if (NS_SUCCEEDED(rv) && !ref.IsEmpty()) {
     NS_UnescapeURL(ref); // XXX may result in random non-ASCII bytes!
-    _hash.Assign(PRUnichar('#'));
+    _hash.Assign(char16_t('#'));
     AppendUTF8toUTF16(ref, _hash);
   }
-  return NS_OK;
 }
 
 void
-Link::ResetLinkState(bool aNotify)
+Link::ResetLinkState(bool aNotify, bool aHasHref)
 {
-  // If we are in our default state, bail early.
-  if (mLinkState == defaultState) {
-    return;
+  nsLinkState defaultState;
+
+  // The default state for links with an href is unvisited.
+  if (aHasHref) {
+    defaultState = eLinkState_Unvisited;
+  } else {
+    defaultState = eLinkState_NotLink;
   }
 
-  Element *element = mElement;
+  // If !mNeedsRegstration, then either we've never registered, or we're
+  // currently registered; in either case, we should remove ourself
+  // from the doc and the history.
+  if (!mNeedsRegistration && mLinkState != eLinkState_NotLink) {
+    nsIDocument *doc = mElement->GetCurrentDoc();
+    if (doc && (mRegistered || mLinkState == eLinkState_Visited)) {
+      // Tell the document to forget about this link if we've registered
+      // with it before.
+      doc->ForgetLink(this);
+    }
 
-  // Tell the document to forget about this link if we were a link before.
-  nsIDocument *doc = element->GetCurrentDoc();
-  if (doc && mLinkState != eLinkState_NotLink) {
-    doc->ForgetLink(this);
+    UnregisterFromHistory();
   }
 
-  UnregisterFromHistory();
+  // If we have an href, we should register with the history.
+  mNeedsRegistration = aHasHref;
+
+  // If we've cached the URI, reset always invalidates it.
+  mCachedURI = nullptr;
+  if (mSearchParams) {
+    mSearchParams->Invalidate();
+  }
 
   // Update our state back to the default.
   mLinkState = defaultState;
 
-  // Get rid of our cached URI.
-  mCachedURI = nullptr;
-
   // We have to be very careful here: if aNotify is false we do NOT
   // want to call UpdateState, because that will call into LinkState()
   // and try to start off loads, etc.  But ResetLinkState is called
   // with aNotify false when things are in inconsistent states, so
   // we'll get confused in that situation.  Instead, just silently
-  // update the link state on mElement.
+  // update the link state on mElement. Since we might have set the
+  // link state to unvisited, make sure to update with that state if
+  // required.
   if (aNotify) {
     mElement->UpdateState(aNotify);
   } else {
-    mElement->UpdateLinkState(nsEventStates());
+    if (mLinkState == eLinkState_Unvisited) {
+      mElement->UpdateLinkState(NS_EVENT_STATE_UNVISITED);
+    } else {
+      mElement->UpdateLinkState(nsEventStates());
+    }
   }
 }
 
@@ -503,7 +557,7 @@ Link::SetHrefAttribute(nsIURI *aURI)
 }
 
 size_t
-Link::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf) const
+Link::SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
 {
   size_t n = 0;
 
@@ -521,5 +575,85 @@ Link::SizeOfExcludingThis(nsMallocSizeOfFun aMallocSizeOf) const
   return n;
 }
 
+URLSearchParams*
+Link::GetSearchParams()
+{
+  CreateSearchParamsIfNeeded();
+  return mSearchParams;
+}
+
+void
+Link::SetSearchParams(URLSearchParams* aSearchParams)
+{
+  if (!aSearchParams) {
+    return;
+  }
+
+  if (!aSearchParams->HasURLAssociated()) {
+    MOZ_ASSERT(aSearchParams->IsValid());
+
+    mSearchParams = aSearchParams;
+    mSearchParams->SetObserver(this);
+  } else {
+    CreateSearchParamsIfNeeded();
+    mSearchParams->CopyFromURLSearchParams(*aSearchParams);
+  }
+
+  nsAutoString search;
+  mSearchParams->Serialize(search);
+  SetSearchInternal(search);
+}
+
+void
+Link::URLSearchParamsUpdated()
+{
+  MOZ_ASSERT(mSearchParams && mSearchParams->IsValid());
+
+  nsString search;
+  mSearchParams->Serialize(search);
+  SetSearchInternal(search);
+}
+
+void
+Link::URLSearchParamsNeedsUpdates()
+{
+  MOZ_ASSERT(mSearchParams);
+
+  nsAutoCString search;
+  nsCOMPtr<nsIURI> uri(GetURI());
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  if (url) {
+    nsresult rv = url->GetQuery(search);
+    if (NS_FAILED(rv)) {
+      NS_WARNING("Failed to get the query from a nsIURL.");
+    }
+  }
+
+  mSearchParams->ParseInput(search);
+}
+
+void
+Link::CreateSearchParamsIfNeeded()
+{
+  if (!mSearchParams) {
+    mSearchParams = new URLSearchParams();
+    mSearchParams->SetObserver(this);
+    mSearchParams->Invalidate();
+  }
+}
+
+void
+Link::Unlink()
+{
+  mSearchParams = nullptr;
+}
+
+void
+Link::Traverse(nsCycleCollectionTraversalCallback &cb)
+{
+  Link* tmp = this;
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mSearchParams);
+}
+
 } // namespace dom
 } // namespace mozilla
