@@ -14,11 +14,10 @@
 #include "nsIURI.h"
 #include "nsNetUtil.h"
 #include "nsIDocShell.h"
-#include "nsIDocShellTreeItem.h"
 #include "nsIStyleSheetLinkingElement.h"
 #include "nsIDOMComment.h"
 #include "nsIDOMCDATASection.h"
-#include "nsDOMDocumentType.h"
+#include "DocumentType.h"
 #include "nsHTMLParts.h"
 #include "nsCRT.h"
 #include "nsCSSStyleSheet.h"
@@ -57,6 +56,11 @@
 #include "mozAutoDocUpdate.h"
 #include "nsMimeTypes.h"
 #include "nsHtml5SVGLoadDispatcher.h"
+#include "nsTextNode.h"
+#include "mozilla/dom/CDATASection.h"
+#include "mozilla/dom/Comment.h"
+#include "mozilla/dom/Element.h"
+#include "mozilla/dom/ProcessingInstruction.h"
 
 using namespace mozilla::dom;
 
@@ -81,9 +85,6 @@ NS_NewXMLContentSink(nsIXMLContentSink** aResult,
     return NS_ERROR_NULL_POINTER;
   }
   nsXMLContentSink* it = new nsXMLContentSink();
-  if (nullptr == it) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
   
   nsCOMPtr<nsIXMLContentSink> kungFuDeathGrip = it;
   nsresult rv = it->Init(aDoc, aURI, aContainer, aChannel);
@@ -450,7 +451,7 @@ nsXMLContentSink::SetParser(nsParserBase* aParser)
 }
 
 nsresult
-nsXMLContentSink::CreateElement(const PRUnichar** aAtts, uint32_t aAttsCount,
+nsXMLContentSink::CreateElement(const char16_t** aAtts, uint32_t aAttsCount,
                                 nsINodeInfo* aNodeInfo, uint32_t aLineNumber,
                                 nsIContent** aResult, bool* aAppendContent,
                                 FromParser aFromParser)
@@ -462,7 +463,7 @@ nsXMLContentSink::CreateElement(const PRUnichar** aAtts, uint32_t aAttsCount,
   nsresult rv = NS_OK;
 
   nsCOMPtr<nsINodeInfo> ni = aNodeInfo;
-  nsCOMPtr<nsIContent> content;
+  nsCOMPtr<Element> content;
   rv = NS_NewElement(getter_AddRefs(content), ni.forget(), aFromParser);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -489,7 +490,7 @@ nsXMLContentSink::CreateElement(const PRUnichar** aAtts, uint32_t aAttsCount,
     }
 
     if (!aNodeInfo->NamespaceEquals(kNameSpaceID_SVG)) {
-      content.swap(*aResult);
+      content.forget(aResult);
 
       return NS_OK;
     }
@@ -510,7 +511,7 @@ nsXMLContentSink::CreateElement(const PRUnichar** aAtts, uint32_t aAttsCount,
     }
   } 
 
-  content.swap(*aResult);
+  content.forget(aResult);
 
   return NS_OK;
 }
@@ -528,10 +529,8 @@ nsXMLContentSink::CloseElement(nsIContent* aContent)
   if ((nodeInfo->NamespaceID() == kNameSpaceID_XHTML &&
        (nodeInfo->NameAtom() == nsGkAtoms::select ||
         nodeInfo->NameAtom() == nsGkAtoms::textarea ||
-#ifdef MOZ_MEDIA
         nodeInfo->NameAtom() == nsGkAtoms::video ||
         nodeInfo->NameAtom() == nsGkAtoms::audio ||
-#endif
         nodeInfo->NameAtom() == nsGkAtoms::object ||
         nodeInfo->NameAtom() == nsGkAtoms::applet))
       || nodeInfo->NameAtom() == nsGkAtoms::title
@@ -724,7 +723,7 @@ nsXMLContentSink::ProcessStyleLink(nsIContent* aElement,
 
     // Do content policy check
     int16_t decision = nsIContentPolicy::ACCEPT;
-    rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_STYLESHEET,
+    rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_XSLT,
                                    url,
                                    mDocument->NodePrincipal(),
                                    aElement,
@@ -803,10 +802,7 @@ nsXMLContentSink::FlushText(bool aReleaseTextNode)
         mTextLength = 0;
       }
     } else {
-      nsCOMPtr<nsIContent> textContent;
-      rv = NS_NewTextNode(getter_AddRefs(textContent),
-                          mNodeInfoManager);
-      NS_ENSURE_SUCCESS(rv, rv);
+      nsRefPtr<nsTextNode> textContent = new nsTextNode(mNodeInfoManager);
 
       mLastTextNode = textContent;
       
@@ -938,8 +934,8 @@ nsXMLContentSink::SetDocElement(int32_t aNameSpaceID,
 }
 
 NS_IMETHODIMP
-nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
-                                     const PRUnichar **aAtts,
+nsXMLContentSink::HandleStartElement(const char16_t *aName,
+                                     const char16_t **aAtts,
                                      uint32_t aAttsCount,
                                      int32_t aIndex,
                                      uint32_t aLineNumber)
@@ -949,8 +945,8 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
 }
 
 nsresult
-nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
-                                     const PRUnichar **aAtts,
+nsXMLContentSink::HandleStartElement(const char16_t *aName,
+                                     const char16_t **aAtts,
                                      uint32_t aAttsCount,
                                      int32_t aIndex,
                                      uint32_t aLineNumber,
@@ -987,7 +983,6 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
   nsCOMPtr<nsINodeInfo> nodeInfo;
   nodeInfo = mNodeInfoManager->GetNodeInfo(localName, prefix, nameSpaceID,
                                            nsIDOMNode::ELEMENT_NODE);
-  NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
 
   result = CreateElement(aAtts, aAttsCount, nodeInfo, aLineNumber,
                          getter_AddRefs(content), &appendContent,
@@ -1031,13 +1026,9 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
   if (nodeInfo->NamespaceID() == kNameSpaceID_XHTML) {
     if (nodeInfo->NameAtom() == nsGkAtoms::input ||
         nodeInfo->NameAtom() == nsGkAtoms::button ||
-        nodeInfo->NameAtom() == nsGkAtoms::menuitem
-#ifdef MOZ_MEDIA
-        ||
+        nodeInfo->NameAtom() == nsGkAtoms::menuitem ||
         nodeInfo->NameAtom() == nsGkAtoms::audio ||
-        nodeInfo->NameAtom() == nsGkAtoms::video
-#endif
-        ) {
+        nodeInfo->NameAtom() == nsGkAtoms::video) {
       content->DoneCreatingElement();
     } else if (nodeInfo->NameAtom() == nsGkAtoms::head && !mCurrentHead) {
       mCurrentHead = content;
@@ -1063,13 +1054,13 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
 }
 
 NS_IMETHODIMP
-nsXMLContentSink::HandleEndElement(const PRUnichar *aName)
+nsXMLContentSink::HandleEndElement(const char16_t *aName)
 {
   return HandleEndElement(aName, true);
 }
 
 nsresult
-nsXMLContentSink::HandleEndElement(const PRUnichar *aName,
+nsXMLContentSink::HandleEndElement(const char16_t *aName,
                                    bool aInterruptable)
 {
   nsresult result = NS_OK;
@@ -1140,23 +1131,20 @@ nsXMLContentSink::HandleEndElement(const PRUnichar *aName,
 }
 
 NS_IMETHODIMP 
-nsXMLContentSink::HandleComment(const PRUnichar *aName)
+nsXMLContentSink::HandleComment(const char16_t *aName)
 {
   FlushText();
 
-  nsCOMPtr<nsIContent> comment;
-  nsresult rv = NS_NewCommentNode(getter_AddRefs(comment), mNodeInfoManager);
-  if (comment) {
-    comment->SetText(nsDependentString(aName), false);
-    rv = AddContentAsLeaf(comment);
-    DidAddContent();
-  }
+  nsRefPtr<Comment> comment = new Comment(mNodeInfoManager);
+  comment->SetText(nsDependentString(aName), false);
+  nsresult rv = AddContentAsLeaf(comment);
+  DidAddContent();
 
   return NS_SUCCEEDED(rv) ? DidProcessATokenImpl() : rv;
 }
 
 NS_IMETHODIMP 
-nsXMLContentSink::HandleCDataSection(const PRUnichar *aData, 
+nsXMLContentSink::HandleCDataSection(const char16_t *aData, 
                                      uint32_t aLength)
 {
   // XSLT doesn't differentiate between text and cdata and wants adjacent
@@ -1167,13 +1155,10 @@ nsXMLContentSink::HandleCDataSection(const PRUnichar *aData,
 
   FlushText();
   
-  nsCOMPtr<nsIContent> cdata;
-  nsresult rv = NS_NewXMLCDATASection(getter_AddRefs(cdata), mNodeInfoManager);
-  if (cdata) {
-    cdata->SetText(aData, aLength, false);
-    rv = AddContentAsLeaf(cdata);
-    DidAddContent();
-  }
+  nsRefPtr<CDATASection> cdata = new CDATASection(mNodeInfoManager);
+  cdata->SetText(aData, aLength, false);
+  nsresult rv = AddContentAsLeaf(cdata);
+  DidAddContent();
 
   return NS_SUCCEEDED(rv) ? DidProcessATokenImpl() : rv;
 }
@@ -1232,14 +1217,14 @@ nsXMLContentSink::HandleDoctypeDecl(const nsAString & aSubset,
 }
 
 NS_IMETHODIMP
-nsXMLContentSink::HandleCharacterData(const PRUnichar *aData, 
+nsXMLContentSink::HandleCharacterData(const char16_t *aData, 
                                       uint32_t aLength)
 {
   return HandleCharacterData(aData, aLength, true);
 }
 
 nsresult
-nsXMLContentSink::HandleCharacterData(const PRUnichar *aData, uint32_t aLength,
+nsXMLContentSink::HandleCharacterData(const char16_t *aData, uint32_t aLength,
                                       bool aInterruptable)
 {
   nsresult rv = NS_OK;
@@ -1251,19 +1236,16 @@ nsXMLContentSink::HandleCharacterData(const PRUnichar *aData, uint32_t aLength,
 }
 
 NS_IMETHODIMP
-nsXMLContentSink::HandleProcessingInstruction(const PRUnichar *aTarget, 
-                                              const PRUnichar *aData)
+nsXMLContentSink::HandleProcessingInstruction(const char16_t *aTarget, 
+                                              const char16_t *aData)
 {
   FlushText();
 
   const nsDependentString target(aTarget);
   const nsDependentString data(aData);
 
-  nsCOMPtr<nsIContent> node;
-
-  nsresult rv = NS_NewXMLProcessingInstruction(getter_AddRefs(node),
-                                               mNodeInfoManager, target, data);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIContent> node =
+    NS_NewXMLProcessingInstruction(mNodeInfoManager, target, data);
 
   nsCOMPtr<nsIStyleSheetLinkingElement> ssle(do_QueryInterface(node));
   if (ssle) {
@@ -1272,7 +1254,7 @@ nsXMLContentSink::HandleProcessingInstruction(const PRUnichar *aTarget,
     mPrettyPrintXML = false;
   }
 
-  rv = AddContentAsLeaf(node);
+  nsresult rv = AddContentAsLeaf(node);
   NS_ENSURE_SUCCESS(rv, rv);
   DidAddContent();
 
@@ -1347,8 +1329,8 @@ nsXMLContentSink::ParsePIData(const nsString &aData, nsString &aHref,
 }
 
 NS_IMETHODIMP
-nsXMLContentSink::HandleXMLDeclaration(const PRUnichar *aVersion,
-                                       const PRUnichar *aEncoding,
+nsXMLContentSink::HandleXMLDeclaration(const char16_t *aVersion,
+                                       const char16_t *aEncoding,
                                        int32_t aStandalone)
 {
   mDocument->SetXMLDeclaration(aVersion, aEncoding, aStandalone);
@@ -1357,8 +1339,8 @@ nsXMLContentSink::HandleXMLDeclaration(const PRUnichar *aVersion,
 }
 
 NS_IMETHODIMP
-nsXMLContentSink::ReportError(const PRUnichar* aErrorText, 
-                              const PRUnichar* aSourceText,
+nsXMLContentSink::ReportError(const char16_t* aErrorText, 
+                              const char16_t* aSourceText,
                               nsIScriptError *aError,
                               bool *_retval)
 {
@@ -1407,17 +1389,17 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
   mContentStack.Clear();
   mNotifyLevel = 0;
 
-  rv = HandleProcessingInstruction(NS_LITERAL_STRING("xml-stylesheet").get(),
-                                   NS_LITERAL_STRING("href=\"chrome://global/locale/intl.css\" type=\"text/css\"").get());
+  rv = HandleProcessingInstruction(MOZ_UTF16("xml-stylesheet"),
+                                   MOZ_UTF16("href=\"chrome://global/locale/intl.css\" type=\"text/css\""));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  const PRUnichar* noAtts[] = { 0, 0 };
+  const char16_t* noAtts[] = { 0, 0 };
 
   NS_NAMED_LITERAL_STRING(errorNs,
                           "http://www.mozilla.org/newlayout/xml/parsererror.xml");
 
   nsAutoString parsererror(errorNs);
-  parsererror.Append((PRUnichar)0xFFFF);
+  parsererror.Append((char16_t)0xFFFF);
   parsererror.AppendLiteral("parsererror");
   
   rv = HandleStartElement(parsererror.get(), noAtts, 0, -1, (uint32_t)-1,
@@ -1428,7 +1410,7 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
   NS_ENSURE_SUCCESS(rv, rv);  
   
   nsAutoString sourcetext(errorNs);
-  sourcetext.Append((PRUnichar)0xFFFF);
+  sourcetext.Append((char16_t)0xFFFF);
   sourcetext.AppendLiteral("sourcetext");
 
   rv = HandleStartElement(sourcetext.get(), noAtts, 0, -1, (uint32_t)-1,
@@ -1450,7 +1432,7 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
 }
 
 nsresult
-nsXMLContentSink::AddAttributes(const PRUnichar** aAtts,
+nsXMLContentSink::AddAttributes(const char16_t** aAtts,
                                 nsIContent* aContent)
 {
   // Add tag attributes to the content attributes
@@ -1472,12 +1454,12 @@ nsXMLContentSink::AddAttributes(const PRUnichar** aAtts,
 #define NS_ACCUMULATION_BUFFER_SIZE 4096
 
 nsresult
-nsXMLContentSink::AddText(const PRUnichar* aText, 
+nsXMLContentSink::AddText(const char16_t* aText, 
                           int32_t aLength)
 {
   // Create buffer when we first need it
   if (0 == mTextSize) {
-    mText = (PRUnichar *) PR_MALLOC(sizeof(PRUnichar) * NS_ACCUMULATION_BUFFER_SIZE);
+    mText = (char16_t *) PR_MALLOC(sizeof(char16_t) * NS_ACCUMULATION_BUFFER_SIZE);
     if (nullptr == mText) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -1500,7 +1482,7 @@ nsXMLContentSink::AddText(const PRUnichar* aText,
       }
       else {
         mTextSize += aLength;
-        mText = (PRUnichar *) PR_REALLOC(mText, sizeof(PRUnichar) * mTextSize);
+        mText = (char16_t *) PR_REALLOC(mText, sizeof(char16_t) * mTextSize);
         if (nullptr == mText) {
           mTextSize = 0;
 
@@ -1513,7 +1495,7 @@ nsXMLContentSink::AddText(const PRUnichar* aText,
     if (amount > aLength) {
       amount = aLength;
     }
-    memcpy(&mText[mTextLength], &aText[offset], sizeof(PRUnichar) * amount);
+    memcpy(&mText[mTextLength], &aText[offset], sizeof(char16_t) * amount);
     mTextLength += amount;
     offset += amount;
     aLength -= amount;
