@@ -16,7 +16,6 @@
 #include "nsCOMPtr.h"
 #include "nsColor.h"
 #include "nsComputedDOMStyle.h"
-#include "nsContentUtils.h"
 #include "nsDebug.h"
 #include "nsDependentSubstring.h"
 #include "nsEditProperty.h"
@@ -105,7 +104,7 @@ void ProcessLengthValue(const nsAString * aInputString, nsAString & aOutputStrin
   aOutputString.Truncate();
   if (aInputString) {
     aOutputString.Append(*aInputString);
-    if (-1 == aOutputString.FindChar(PRUnichar('%'))) {
+    if (-1 == aOutputString.FindChar(char16_t('%'))) {
       aOutputString.AppendLiteral("px");
     }
   }
@@ -183,45 +182,6 @@ void ProcessMarginRightValue(const nsAString * aInputString, nsAString & aOutput
   }
 }
 
-static
-void ProcessFontSizeValue(const nsAString* aInputString, nsAString& aOutputString,
-                          const char* aDefaultValueString,
-                          const char* aPrependString, const char* aAppendString)
-{
-  aOutputString.Truncate();
-  if (aInputString) {
-    int32_t size = nsContentUtils::ParseLegacyFontSize(*aInputString);
-    switch (size) {
-      case 0:
-        // Didn't parse
-        return;
-      case 1:
-        aOutputString.AssignLiteral("x-small");
-        return;
-      case 2:
-        aOutputString.AssignLiteral("small");
-        return;
-      case 3:
-        aOutputString.AssignLiteral("medium");
-        return;
-      case 4:
-        aOutputString.AssignLiteral("large");
-        return;
-      case 5:
-        aOutputString.AssignLiteral("x-large");
-        return;
-      case 6:
-        aOutputString.AssignLiteral("xx-large");
-        return;
-      case 7:
-        // No corresponding CSS size
-        return;
-      default:
-        NS_NOTREACHED("Unexpected return value from ParseLegacyFontSize");
-    }
-  }
-}
-
 const nsHTMLCSSUtils::CSSEquivTable boldEquivTable[] = {
   { nsHTMLCSSUtils::eCSSEditableProperty_font_weight, ProcessBValue, nullptr, nullptr, nullptr, true, false },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
@@ -257,11 +217,6 @@ const nsHTMLCSSUtils::CSSEquivTable fontFaceEquivTable[] = {
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
-const nsHTMLCSSUtils::CSSEquivTable fontSizeEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_font_size, ProcessFontSizeValue, nullptr, nullptr, nullptr, true, false },
-  { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
-};
-
 const nsHTMLCSSUtils::CSSEquivTable bgcolorEquivTable[] = {
   { nsHTMLCSSUtils::eCSSEditableProperty_background_color, ProcessSameValue, nullptr, nullptr, nullptr, true, false },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
@@ -347,21 +302,19 @@ nsHTMLCSSUtils::~nsHTMLCSSUtils()
 bool
 nsHTMLCSSUtils::IsCSSEditableProperty(nsIDOMNode* aNode,
                                       nsIAtom* aProperty,
-                                      const nsAString* aAttribute,
-                                      const nsAString* aValue)
+                                      const nsAString* aAttribute)
 {
   NS_ASSERTION(aNode, "Shouldn't you pass aNode? - Bug 214025");
 
   nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
   NS_ENSURE_TRUE(content, false);
-  return IsCSSEditableProperty(content, aProperty, aAttribute, aValue);
+  return IsCSSEditableProperty(content, aProperty, aAttribute);
 }
 
 bool
 nsHTMLCSSUtils::IsCSSEditableProperty(nsIContent* aNode,
                                       nsIAtom* aProperty,
-                                      const nsAString* aAttribute,
-                                      const nsAString* aValue)
+                                      const nsAString* aAttribute)
 {
   MOZ_ASSERT(aNode);
 
@@ -387,16 +340,6 @@ nsHTMLCSSUtils::IsCSSEditableProperty(nsIContent* aNode,
     return true;
   }
 
-  // FONT SIZE doesn't work if the value is 7
-  if (nsEditProperty::font == aProperty && aAttribute &&
-      aAttribute->EqualsLiteral("size")) {
-    if (!aValue || aValue->IsEmpty()) {
-      return true;
-    }
-    int32_t size = nsContentUtils::ParseLegacyFontSize(*aValue);
-    return size && size != 7;
-  }
-
   // ALIGN attribute on elements supporting it
   if (aAttribute && (aAttribute->EqualsLiteral("align")) &&
       (nsEditProperty::div == tagName
@@ -645,7 +588,6 @@ nsHTMLCSSUtils::GetComputedStyle(dom::Element* aElement)
   NS_ENSURE_TRUE(doc, nullptr);
 
   nsIPresShell* presShell = doc->GetShell();
-  NS_ASSERTION(presShell, "Trying to compute style without PresShell");
   NS_ENSURE_TRUE(presShell, nullptr);
 
   nsRefPtr<nsComputedDOMStyle> style =
@@ -733,27 +675,27 @@ nsHTMLCSSUtils::ParseLength(const nsAString & aString, float * aValue, nsIAtom *
   float a = 10.0f , b = 1.0f, value = 0;
   int8_t sign = 1;
   int32_t i = 0, j = aString.Length();
-  PRUnichar c;
+  char16_t c;
   bool floatingPointFound = false;
   c = *iter;
-  if (PRUnichar('-') == c) { sign = -1; iter++; i++; }
-  else if (PRUnichar('+') == c) { iter++; i++; }
+  if (char16_t('-') == c) { sign = -1; iter++; i++; }
+  else if (char16_t('+') == c) { iter++; i++; }
   while (i < j) {
     c = *iter;
-    if ((PRUnichar('0') == c) ||
-        (PRUnichar('1') == c) ||
-        (PRUnichar('2') == c) ||
-        (PRUnichar('3') == c) ||
-        (PRUnichar('4') == c) ||
-        (PRUnichar('5') == c) ||
-        (PRUnichar('6') == c) ||
-        (PRUnichar('7') == c) ||
-        (PRUnichar('8') == c) ||
-        (PRUnichar('9') == c)) {
-      value = (value * a) + (b * (c - PRUnichar('0')));
+    if ((char16_t('0') == c) ||
+        (char16_t('1') == c) ||
+        (char16_t('2') == c) ||
+        (char16_t('3') == c) ||
+        (char16_t('4') == c) ||
+        (char16_t('5') == c) ||
+        (char16_t('6') == c) ||
+        (char16_t('7') == c) ||
+        (char16_t('8') == c) ||
+        (char16_t('9') == c)) {
+      value = (value * a) + (b * (c - char16_t('0')));
       b = b / 10 * a;
     }
-    else if (!floatingPointFound && (PRUnichar('.') == c)) {
+    else if (!floatingPointFound && (char16_t('.') == c)) {
       floatingPointFound = true;
       a = 1.0f; b = 0.1f;
     }
@@ -762,7 +704,7 @@ nsHTMLCSSUtils::ParseLength(const nsAString & aString, float * aValue, nsIAtom *
     i++;
   }
   *aValue = value * sign;
-  *aUnit = NS_NewAtom(StringTail(aString, j-i)); 
+  *aUnit = NS_NewAtom(StringTail(aString, j-i)).get();
 }
 
 void
@@ -908,9 +850,6 @@ nsHTMLCSSUtils::GenerateCSSDeclarationsFromHTMLStyle(dom::Element* aElement,
     } else if (nsEditProperty::font == aHTMLProperty &&
                aAttribute->EqualsLiteral("face")) {
       equivTable = fontFaceEquivTable;
-    } else if (nsEditProperty::font == aHTMLProperty &&
-               aAttribute->EqualsLiteral("size")) {
-      equivTable = fontSizeEquivTable;
     } else if (aAttribute->EqualsLiteral("bgcolor")) {
       equivTable = bgcolorEquivTable;
     } else if (aAttribute->EqualsLiteral("background")) {
@@ -989,8 +928,7 @@ nsHTMLCSSUtils::SetCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
 {
   nsCOMPtr<dom::Element> element = do_QueryInterface(aNode);
   *aCount = 0;
-  if (!element || !IsCSSEditableProperty(element, aHTMLProperty,
-                                         aAttribute, aValue)) {
+  if (!element || !IsCSSEditableProperty(element, aHTMLProperty, aAttribute)) {
     return NS_OK;
   }
 
@@ -1024,7 +962,22 @@ nsHTMLCSSUtils::RemoveCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
                                                bool aSuppressTransaction)
 {
   nsCOMPtr<dom::Element> element = do_QueryInterface(aNode);
-  if (!element || !IsCSSEditableProperty(element, aHTMLProperty, aAttribute)) {
+  NS_ENSURE_TRUE(element, NS_OK);
+
+  return RemoveCSSEquivalentToHTMLStyle(element, aHTMLProperty, aAttribute,
+                                        aValue, aSuppressTransaction);
+}
+
+nsresult
+nsHTMLCSSUtils::RemoveCSSEquivalentToHTMLStyle(dom::Element* aElement,
+                                               nsIAtom* aHTMLProperty,
+                                               const nsAString* aAttribute,
+                                               const nsAString* aValue,
+                                               bool aSuppressTransaction)
+{
+  MOZ_ASSERT(aElement);
+
+  if (!IsCSSEditableProperty(aElement, aHTMLProperty, aAttribute)) {
     return NS_OK;
   }
 
@@ -1034,11 +987,11 @@ nsHTMLCSSUtils::RemoveCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
   // Find the CSS equivalence to the HTML style
   nsTArray<nsIAtom*> cssPropertyArray;
   nsTArray<nsString> cssValueArray;
-  GenerateCSSDeclarationsFromHTMLStyle(element, aHTMLProperty, aAttribute,
+  GenerateCSSDeclarationsFromHTMLStyle(aElement, aHTMLProperty, aAttribute,
                                        aValue, cssPropertyArray, cssValueArray,
                                        true);
 
-  nsCOMPtr<nsIDOMElement> domElement = do_QueryInterface(element);
+  nsCOMPtr<nsIDOMElement> domElement = do_QueryInterface(aElement);
   // remove the individual CSS inline styles
   int32_t count = cssPropertyArray.Length();
   for (int32_t index = 0; index < count; index++) {
@@ -1066,8 +1019,7 @@ nsHTMLCSSUtils::GetCSSEquivalentToHTMLInlineStyleSet(nsINode* aNode,
   nsCOMPtr<dom::Element> theElement = GetElementContainerOrSelf(aNode);
   NS_ENSURE_TRUE(theElement, NS_ERROR_NULL_POINTER);
 
-  if (!theElement || !IsCSSEditableProperty(theElement, aHTMLProperty,
-                                            aAttribute, &aValueString)) {
+  if (!theElement || !IsCSSEditableProperty(theElement, aHTMLProperty, aAttribute)) {
     return NS_OK;
   }
 
@@ -1087,7 +1039,7 @@ nsHTMLCSSUtils::GetCSSEquivalentToHTMLInlineStyleSet(nsINode* aNode,
     NS_ENSURE_SUCCESS(res, res);
     // append the value to aValueString (possibly with a leading whitespace)
     if (index) {
-      aValueString.Append(PRUnichar(' '));
+      aValueString.Append(char16_t(' '));
     }
     aValueString.Append(valueString);
   }
@@ -1156,7 +1108,7 @@ nsHTMLCSSUtils::IsCSSEquivalentToHTMLInlineStyleSet(nsIDOMNode *aNode,
         int32_t weight = 0;
         nsresult errorCode;
         nsAutoString value(valueString);
-        weight = value.ToInteger(&errorCode, 10);
+        weight = value.ToInteger(&errorCode);
         if (400 < weight) {
           aIsSet = true;
           valueString.AssignLiteral("bold");
@@ -1214,7 +1166,7 @@ nsHTMLCSSUtils::IsCSSEquivalentToHTMLInlineStyleSet(nsIDOMNode *aNode,
             tmpStr.AppendInt(NS_GET_B(rgba), 10);
             htmlColor.Append(tmpStr);
 
-            htmlColor.Append(PRUnichar(')'));
+            htmlColor.Append(char16_t(')'));
           }
 
           aIsSet = htmlColor.Equals(valueString,
@@ -1229,8 +1181,8 @@ nsHTMLCSSUtils::IsCSSEquivalentToHTMLInlineStyleSet(nsIDOMNode *aNode,
     } else if (nsEditProperty::font == aHTMLProperty && aHTMLAttribute &&
                aHTMLAttribute->EqualsLiteral("face")) {
       if (!htmlValueString.IsEmpty()) {
-        const PRUnichar commaSpace[] = { PRUnichar(','), PRUnichar(' '), 0 };
-        const PRUnichar comma[] = { PRUnichar(','), 0 };
+        const char16_t commaSpace[] = { char16_t(','), char16_t(' '), 0 };
+        const char16_t comma[] = { char16_t(','), 0 };
         htmlValueString.ReplaceSubstring(commaSpace, comma);
         nsAutoString valueStringNorm(valueString);
         valueStringNorm.ReplaceSubstring(commaSpace, comma);
@@ -1244,19 +1196,6 @@ nsHTMLCSSUtils::IsCSSEquivalentToHTMLInlineStyleSet(nsIDOMNode *aNode,
                  !valueStringLower.EqualsLiteral("serif");
       }
       return NS_OK;
-    } else if (nsEditProperty::font == aHTMLProperty && aHTMLAttribute &&
-               aHTMLAttribute->EqualsLiteral("size")) {
-      if (htmlValueString.IsEmpty()) {
-        aIsSet = true;
-      } else {
-        int32_t size = nsContentUtils::ParseLegacyFontSize(htmlValueString);
-        aIsSet = (size == 1 && valueString.EqualsLiteral("x-small")) ||
-                 (size == 2 && valueString.EqualsLiteral("small")) ||
-                 (size == 3 && valueString.EqualsLiteral("medium")) ||
-                 (size == 4 && valueString.EqualsLiteral("large")) ||
-                 (size == 5 && valueString.EqualsLiteral("x-large")) ||
-                 (size == 6 && valueString.EqualsLiteral("xx-large"));
-      }
     } else if (aHTMLAttribute && aHTMLAttribute->EqualsLiteral("align")) {
       aIsSet = true;
     } else {
@@ -1277,7 +1216,7 @@ nsHTMLCSSUtils::IsCSSEquivalentToHTMLInlineStyleSet(nsIDOMNode *aNode,
     if (nsEditProperty::u == aHTMLProperty || nsEditProperty::strike == aHTMLProperty) {
       // unfortunately, the value of the text-decoration property is not inherited.
       // that means that we have to look at ancestors of node to see if they are underlined
-      node = node->GetElementParent();  // set to null if it's not a dom element
+      node = node->GetParentElement();  // set to null if it's not a dom element
     }
   } while ((nsEditProperty::u == aHTMLProperty || nsEditProperty::strike == aHTMLProperty) &&
            !aIsSet && node);
@@ -1300,36 +1239,35 @@ nsHTMLCSSUtils::IsCSSPrefChecked()
 // specified CSS declarations in the STYLE attribute 
 // The answer is always negative if at least one of them carries an ID or a class
 bool
-nsHTMLCSSUtils::ElementsSameStyle(dom::Element* aFirstNode,
-                                  dom::Element* aSecondNode)
+nsHTMLCSSUtils::ElementsSameStyle(nsIDOMNode *aFirstNode, nsIDOMNode *aSecondNode)
 {
-  MOZ_ASSERT(aFirstNode && aSecondNode);
-  return ElementsSameStyle(aFirstNode->AsDOMNode(), aSecondNode->AsDOMNode());
+  nsCOMPtr<dom::Element> firstElement  = do_QueryInterface(aFirstNode);
+  nsCOMPtr<dom::Element> secondElement = do_QueryInterface(aSecondNode);
+
+  NS_ASSERTION((firstElement && secondElement), "Non element nodes passed to ElementsSameStyle.");
+  NS_ENSURE_TRUE(firstElement, false);
+  NS_ENSURE_TRUE(secondElement, false);
+
+  return ElementsSameStyle(firstElement, secondElement);
 }
 
 bool
-nsHTMLCSSUtils::ElementsSameStyle(nsIDOMNode *aFirstNode, nsIDOMNode *aSecondNode)
+nsHTMLCSSUtils::ElementsSameStyle(dom::Element* aFirstElement,
+                                  dom::Element* aSecondElement)
 {
-  nsresult res;
-  nsCOMPtr<nsIDOMElement> firstElement  = do_QueryInterface(aFirstNode);
-  nsCOMPtr<nsIDOMElement> secondElement = do_QueryInterface(aSecondNode);
+  MOZ_ASSERT(aFirstElement);
+  MOZ_ASSERT(aSecondElement);
 
-  NS_ASSERTION((firstElement && secondElement), "Non element nodes passed to ElementsSameStyle.");
-
-  nsAutoString firstID, secondID;
-  bool isFirstIDSet, isSecondIDSet;
-  res = mHTMLEditor->GetAttributeValue(firstElement,  NS_LITERAL_STRING("id"), firstID,  &isFirstIDSet);
-  res = mHTMLEditor->GetAttributeValue(secondElement, NS_LITERAL_STRING("id"), secondID, &isSecondIDSet);
-  if (isFirstIDSet || isSecondIDSet) {
+  if (aFirstElement->HasAttr(kNameSpaceID_None, nsGkAtoms::id) ||
+      aSecondElement->HasAttr(kNameSpaceID_None, nsGkAtoms::id)) {
     // at least one of the spans carries an ID ; suspect a CSS rule applies to it and
     // refuse to merge the nodes
     return false;
   }
 
   nsAutoString firstClass, secondClass;
-  bool isFirstClassSet, isSecondClassSet;
-  res = mHTMLEditor->GetAttributeValue(firstElement,  NS_LITERAL_STRING("class"), firstClass,  &isFirstClassSet);
-  res = mHTMLEditor->GetAttributeValue(secondElement, NS_LITERAL_STRING("class"), secondClass, &isSecondClassSet);
+  bool isFirstClassSet = aFirstElement->GetAttr(kNameSpaceID_None, nsGkAtoms::_class, firstClass);
+  bool isSecondClassSet = aSecondElement->GetAttr(kNameSpaceID_None, nsGkAtoms::_class, secondClass);
   if (isFirstClassSet && isSecondClassSet) {
     // both spans carry a class, let's compare them
     if (!firstClass.Equals(secondClass)) {
@@ -1341,32 +1279,35 @@ nsHTMLCSSUtils::ElementsSameStyle(nsIDOMNode *aFirstNode, nsIDOMNode *aSecondNod
       // need to discuss this issue before any modification.
       return false;
     }
-  }
-  else if (isFirstClassSet || isSecondClassSet) {
+  } else if (isFirstClassSet || isSecondClassSet) {
     // one span only carries a class, early way out
     return false;
   }
 
   nsCOMPtr<nsIDOMCSSStyleDeclaration> firstCSSDecl, secondCSSDecl;
   uint32_t firstLength, secondLength;
-  res = GetInlineStyles(firstElement,  getter_AddRefs(firstCSSDecl),  &firstLength);
-  if (NS_FAILED(res) || !firstCSSDecl) return false;
-  res = GetInlineStyles(secondElement, getter_AddRefs(secondCSSDecl), &secondLength);
-  if (NS_FAILED(res) || !secondCSSDecl) return false;
+  nsresult rv = GetInlineStyles(aFirstElement,  getter_AddRefs(firstCSSDecl),  &firstLength);
+  if (NS_FAILED(rv) || !firstCSSDecl) {
+    return false;
+  }
+  rv = GetInlineStyles(aSecondElement, getter_AddRefs(secondCSSDecl), &secondLength);
+  if (NS_FAILED(rv) || !secondCSSDecl) {
+    return false;
+  }
 
   if (firstLength != secondLength) {
     // early way out if we can
     return false;
   }
-  else if (0 == firstLength) {
+
+  if (!firstLength) {
     // no inline style !
     return true;
   }
 
-  uint32_t i;
   nsAutoString propertyNameString;
   nsAutoString firstValue, secondValue;
-  for (i=0; i<firstLength; i++) {
+  for (uint32_t i = 0; i < firstLength; i++) {
     firstCSSDecl->Item(i, propertyNameString);
     firstCSSDecl->GetPropertyValue(propertyNameString, firstValue);
     secondCSSDecl->GetPropertyValue(propertyNameString, secondValue);
@@ -1374,7 +1315,7 @@ nsHTMLCSSUtils::ElementsSameStyle(nsIDOMNode *aFirstNode, nsIDOMNode *aSecondNod
       return false;
     }
   }
-  for (i=0; i<secondLength; i++) {
+  for (uint32_t i = 0; i < secondLength; i++) {
     secondCSSDecl->Item(i, propertyNameString);
     secondCSSDecl->GetPropertyValue(propertyNameString, secondValue);
     firstCSSDecl->GetPropertyValue(propertyNameString, firstValue);
@@ -1387,7 +1328,23 @@ nsHTMLCSSUtils::ElementsSameStyle(nsIDOMNode *aFirstNode, nsIDOMNode *aSecondNod
 }
 
 nsresult
-nsHTMLCSSUtils::GetInlineStyles(nsIDOMElement *aElement,
+nsHTMLCSSUtils::GetInlineStyles(dom::Element* aElement,
+                                nsIDOMCSSStyleDeclaration** aCssDecl,
+                                uint32_t* aLength)
+{
+  return GetInlineStyles(static_cast<nsISupports*>(aElement), aCssDecl, aLength);
+}
+
+nsresult
+nsHTMLCSSUtils::GetInlineStyles(nsIDOMElement* aElement,
+                                nsIDOMCSSStyleDeclaration** aCssDecl,
+                                uint32_t* aLength)
+{
+  return GetInlineStyles(static_cast<nsISupports*>(aElement), aCssDecl, aLength);
+}
+
+nsresult
+nsHTMLCSSUtils::GetInlineStyles(nsISupports *aElement,
                                 nsIDOMCSSStyleDeclaration **aCssDecl,
                                 uint32_t *aLength)
 {
@@ -1395,8 +1352,11 @@ nsHTMLCSSUtils::GetInlineStyles(nsIDOMElement *aElement,
   *aLength = 0;
   nsCOMPtr<nsIDOMElementCSSInlineStyle> inlineStyles = do_QueryInterface(aElement);
   NS_ENSURE_TRUE(inlineStyles, NS_ERROR_NULL_POINTER);
+
   nsresult res = inlineStyles->GetStyle(aCssDecl);
-  if (NS_FAILED(res) || !aCssDecl) return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_SUCCESS(res, NS_ERROR_NULL_POINTER);
+  MOZ_ASSERT(*aCssDecl);
+
   (*aCssDecl)->GetLength(aLength);
   return NS_OK;
 }
@@ -1422,7 +1382,7 @@ nsHTMLCSSUtils::GetElementContainerOrSelf(nsINode* aNode)
   nsINode* node = aNode;
   // Loop until we find an element.
   while (node && !node->IsElement()) {
-    node = node->GetNodeParent();
+    node = node->GetParentNode();
   }
 
   NS_ENSURE_TRUE(node, nullptr);
