@@ -187,72 +187,7 @@ class AutoSetHandlingSignal
     }
 };
 
-// For platforms that install a single, process-wide signal handler (Unix and
-// Windows), the InstallSignalHandlersMutex prevents races between JSRuntimes
-// installing signal handlers.
-#if !defined(XP_MACOSX)
-# if defined(JS_THREADSAFE)
-#  include "jslock.h"
-
-namespace {
-
-class InstallSignalHandlersMutex
-{
-    PRLock *mutex_;
-
-  public:
-    InstallSignalHandlersMutex() {
-        mutex_ = PR_NewLock();
-        if (!mutex_)
-            MOZ_CRASH();
-    }
-    ~InstallSignalHandlersMutex() {
-        PR_DestroyLock(mutex_);
-    }
-    class Lock {
-        static bool sHandlersInstalled;
-      public:
-        Lock();
-        ~Lock();
-        bool handlersInstalled() const { return sHandlersInstalled; }
-        void setHandlersInstalled() { sHandlersInstalled = true; }
-    };
-} signalMutex;
-
-} /* anonymous namespace */
-
-bool InstallSignalHandlersMutex::Lock::sHandlersInstalled = false;
-
-InstallSignalHandlersMutex::Lock::Lock()
-{
-    PR_Lock(signalMutex.mutex_);
-}
-
-InstallSignalHandlersMutex::Lock::~Lock()
-{
-    PR_Unlock(signalMutex.mutex_);
-}
-# else  // JS_THREADSAFE
-namespace {
-
-struct InstallSignalHandlersMutex
-{
-    class Lock {
-        static bool sHandlersInstalled;
-      public:
-        Lock() { (void)this; }
-        bool handlersInstalled() const { return sHandlersInstalled; }
-        void setHandlersInstalled() { sHandlersInstalled = true; }
-    };
-};
-
-} /* anonymous namespace */
-
-bool InstallSignalHandlersMutex::Lock::sHandlersInstalled = false;
-# endif  // JS_THREADSAFE
-#endif   // !XP_MACOSX
-
-#if defined(JS_CPU_X64)
+#if defined(JS_CODEGEN_X64)
 template <class T>
 static void
 SetXMMRegToNaN(bool isFloat32, T *xmm_reg)
@@ -315,7 +250,7 @@ LookupHeapAccess(const AsmJSModule &module, uint8_t *pc)
 # include <sys/ucontext.h> // for ucontext_t, mcontext_t
 #endif
 
-#if defined(JS_CPU_X64)
+#if defined(JS_CODEGEN_X64)
 # if defined(__DragonFly__)
 #  include <machine/npx.h> // for union savefpu
 # elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
@@ -397,6 +332,29 @@ static bool IsSignalHandlingBroken() { return false; }
 # define PC_sig(p) R15_sig(p)
 #endif
 
+static bool
+HandleSimulatorInterrupt(JSRuntime *rt, AsmJSActivation *activation, void *faultingAddress)
+{
+    // If the ARM simulator is enabled, the pc is in the simulator C++ code and
+    // not in the generated code, so we check the simulator's pc manually. Also
+    // note that we can't simply use simulator->set_pc() here because the
+    // simulator could be in the middle of an instruction. On ARM, the signal
+    // handlers are currently only used for Odin code, see bug 964258.
+
+#ifdef JS_ARM_SIMULATOR
+    const AsmJSModule &module = activation->module();
+    if (module.containsPC((void *)rt->mainThread.simulator()->get_pc()) &&
+        module.containsPC(faultingAddress))
+    {
+        activation->setResumePC(nullptr);
+        int32_t nextpc = int32_t(module.operationCallbackExit());
+        rt->mainThread.simulator()->set_resume_pc(nextpc);
+        return true;
+    }
+#endif
+    return false;
+}
+
 #if !defined(XP_MACOSX)
 static uint8_t **
 ContextToPC(CONTEXT *context)
@@ -405,7 +363,7 @@ ContextToPC(CONTEXT *context)
     return reinterpret_cast<uint8_t**>(&PC_sig(context));
 }
 
-# if defined(JS_CPU_X64)
+# if defined(JS_CODEGEN_X64)
 static void
 SetRegisterToCoercedUndefined(CONTEXT *context, bool isFloat32, AnyRegister reg)
 {
@@ -451,7 +409,7 @@ SetRegisterToCoercedUndefined(CONTEXT *context, bool isFloat32, AnyRegister reg)
         }
     }
 }
-# endif  // JS_CPU_X64
+# endif  // JS_CODEGEN_X64
 #endif   // !XP_MACOSX
 
 #if defined(XP_WIN)
@@ -506,7 +464,7 @@ HandleException(PEXCEPTION_POINTERS exception)
         return true;
     }
 
-# if defined(JS_CPU_X64)
+# if defined(JS_CODEGEN_X64)
     // These checks aren't necessary, but, since we can, check anyway to make
     // sure we aren't covering up a real bug.
     if (!module.maybeHeap() ||
@@ -555,7 +513,7 @@ AsmJSExceptionHandler(LPEXCEPTION_POINTERS exception)
 static uint8_t **
 ContextToPC(x86_thread_state_t &state)
 {
-# if defined(JS_CPU_X64)
+# if defined(JS_CODEGEN_X64)
     JS_STATIC_ASSERT(sizeof(state.uts.ts64.__rip) == sizeof(void*));
     return reinterpret_cast<uint8_t**>(&state.uts.ts64.__rip);
 # else
@@ -564,7 +522,7 @@ ContextToPC(x86_thread_state_t &state)
 # endif
 }
 
-# if defined(JS_CPU_X64)
+# if defined(JS_CODEGEN_X64)
 static bool
 SetRegisterToCoercedUndefined(mach_port_t rtThread, x86_thread_state64_t &state,
                               const AsmJSHeapAccess &heapAccess)
@@ -686,6 +644,11 @@ HandleMachException(JSRuntime *rt, const ExceptionRequest &request)
         return false;
 
     const AsmJSModule &module = activation->module();
+    if (HandleSimulatorInterrupt(rt, activation, faultingAddress)) {
+        mprotect(module.codeBase(), module.functionBytes(), PROT_EXEC);
+        return true;
+    }
+
     if (!module.containsPC(pc))
         return false;
 
@@ -704,7 +667,7 @@ HandleMachException(JSRuntime *rt, const ExceptionRequest &request)
         return kret == KERN_SUCCESS;
     }
 
-# if defined(JS_CPU_X64)
+# if defined(JS_CODEGEN_X64)
     // These checks aren't necessary, but, since we can, check anyway to make
     // sure we aren't covering up a real bug.
     if (!module.maybeHeap() ||
@@ -747,7 +710,7 @@ static const mach_msg_id_t sExceptionId = 2405;
 // The choice of id here is arbitrary, the only constraint is that sQuitId != sExceptionId.
 static const mach_msg_id_t sQuitId = 42;
 
-void *
+void
 AsmJSMachExceptionHandlerThread(void *threadArg)
 {
     JSRuntime *rt = reinterpret_cast<JSRuntime*>(threadArg);
@@ -798,8 +761,6 @@ AsmJSMachExceptionHandlerThread(void *threadArg)
         mach_msg(&reply.Head, MACH_SEND_MSG, sizeof(reply), 0, MACH_PORT_NULL,
                  MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
     }
-
-    return nullptr;
 }
 
 AsmJSMachExceptionHandler::AsmJSMachExceptionHandler()
@@ -811,6 +772,7 @@ AsmJSMachExceptionHandler::AsmJSMachExceptionHandler()
 void
 AsmJSMachExceptionHandler::uninstall()
 {
+#ifdef JS_THREADSAFE
     if (installed_) {
         thread_port_t thread = mach_thread_self();
         kern_return_t kret = thread_set_exception_ports(thread,
@@ -840,7 +802,7 @@ AsmJSMachExceptionHandler::uninstall()
         }
 
         // Wait for the handler thread to complete before deallocating the port.
-        pthread_join(thread_, nullptr);
+        PR_JoinThread(thread_);
         thread_ = nullptr;
     }
     if (port_ != MACH_PORT_NULL) {
@@ -848,11 +810,15 @@ AsmJSMachExceptionHandler::uninstall()
         JS_ASSERT(kret == KERN_SUCCESS);
         port_ = MACH_PORT_NULL;
     }
+#else
+    JS_ASSERT(!installed_);
+#endif
 }
 
 bool
 AsmJSMachExceptionHandler::install(JSRuntime *rt)
 {
+#ifdef JS_THREADSAFE
     JS_ASSERT(!installed());
     kern_return_t kret;
     mach_port_t thread;
@@ -866,7 +832,9 @@ AsmJSMachExceptionHandler::install(JSRuntime *rt)
         goto error;
 
     // Create a thread to block on reading port_.
-    if (pthread_create(&thread_, nullptr, AsmJSMachExceptionHandlerThread, rt))
+    thread_ = PR_CreateThread(PR_USER_THREAD, AsmJSMachExceptionHandlerThread, rt,
+                              PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD, PR_JOINABLE_THREAD, 0);
+    if (!thread_)
         goto error;
 
     // Direct exceptions on this thread to port_ (and thus our handler thread).
@@ -890,6 +858,9 @@ AsmJSMachExceptionHandler::install(JSRuntime *rt)
   error:
     uninstall();
     return false;
+#else
+    return false;
+#endif
 }
 
 #else  // If not Windows or Mac, assume Unix
@@ -920,6 +891,11 @@ HandleSignal(int signum, siginfo_t *info, void *ctx)
         return false;
 
     const AsmJSModule &module = activation->module();
+    if (HandleSimulatorInterrupt(rt, activation, faultingAddress)) {
+        mprotect(module.codeBase(), module.functionBytes(), PROT_EXEC);
+        return true;
+    }
+
     if (!module.containsPC(pc))
         return false;
 
@@ -935,7 +911,7 @@ HandleSignal(int signum, siginfo_t *info, void *ctx)
         return true;
     }
 
-# if defined(JS_CPU_X64)
+# if defined(JS_CODEGEN_X64)
     // These checks aren't necessary, but, since we can, check anyway to make
     // sure we aren't covering up a real bug.
     if (!module.maybeHeap() ||
@@ -993,6 +969,10 @@ AsmJSFaultHandler(int signum, siginfo_t *info, void *context)
 }
 #endif
 
+#if !defined(XP_MACOSX)
+static bool sHandlersInstalled = false;
+#endif
+
 bool
 js::EnsureAsmJSSignalHandlersInstalled(JSRuntime *rt)
 {
@@ -1005,8 +985,7 @@ js::EnsureAsmJSSignalHandlersInstalled(JSRuntime *rt)
 #else
     // Assume Windows or Unix. For these platforms, there is a single,
     // process-wide signal handler installed. Take care to only install it once.
-    InstallSignalHandlersMutex::Lock lock;
-    if (lock.handlersInstalled())
+    if (sHandlersInstalled)
         return true;
 
 # if defined(XP_WIN)
@@ -1024,7 +1003,7 @@ js::EnsureAsmJSSignalHandlersInstalled(JSRuntime *rt)
         return false;
 # endif
 
-    lock.setHandlersInstalled();
+    sHandlersInstalled = true;
 #endif
     return true;
 }
