@@ -18,15 +18,13 @@
 
 #include <stdlib.h>
 #include "nsHostResolver.h"
-#include "nsNetError.h"
+#include "nsError.h"
 #include "nsISupportsBase.h"
 #include "nsISupportsUtils.h"
 #include "nsAutoPtr.h"
-#include "pratom.h"
 #include "prthread.h"
 #include "prerror.h"
 #include "prtime.h"
-#include "prlong.h"
 #include "prlog.h"
 #include "pldhash.h"
 #include "plstr.h"
@@ -34,11 +32,12 @@
 #include "nsThreadUtils.h"
 
 #include "mozilla/HashFunctions.h"
-#include "mozilla/FunctionTimer.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/VisualEventTracer.h"
 
 using namespace mozilla;
+using namespace mozilla::net;
 
 //----------------------------------------------------------------------------
 
@@ -66,7 +65,7 @@ PR_STATIC_ASSERT (HighThreadThreshold <= MAX_RESOLVER_THREADS);
 //----------------------------------------------------------------------------
 
 #if defined(PR_LOGGING)
-static PRLogModuleInfo *gHostResolverLog = nsnull;
+static PRLogModuleInfo *gHostResolverLog = nullptr;
 #define LOG(args) PR_LOG(gHostResolverLog, PR_LOG_DEBUG, args)
 #else
 #define LOG(args)
@@ -86,17 +85,6 @@ MoveCList(PRCList &from, PRCList &to)
     }             
 }
 
-static PRUint32
-NowInMinutes()
-{
-    PRTime now = PR_Now(), minutes, factor;
-    LL_I2L(factor, 60 * PR_USEC_PER_SEC);
-    LL_DIV(minutes, now, factor);
-    PRUint32 result;
-    LL_L2UI(result, minutes);
-    return result;
-}
-
 //----------------------------------------------------------------------------
 
 #if defined(RES_RETRY_ON_FAILURE)
@@ -129,7 +117,7 @@ public:
         if (PR_IntervalToSeconds(PR_IntervalNow() - mLastReset) < 1)
             return false;
 
-        LOG(("calling res_ninit\n"));
+        LOG(("Calling 'res_ninit'.\n"));
 
         mLastReset = PR_IntervalNow();
         return (res_ninit(&_res) == 0);
@@ -143,6 +131,26 @@ private:
 
 //----------------------------------------------------------------------------
 
+static inline bool
+IsHighPriority(uint16_t flags)
+{
+    return !(flags & (nsHostResolver::RES_PRIORITY_LOW | nsHostResolver::RES_PRIORITY_MEDIUM));
+}
+
+static inline bool
+IsMediumPriority(uint16_t flags)
+{
+    return flags & nsHostResolver::RES_PRIORITY_MEDIUM;
+}
+
+static inline bool
+IsLowPriority(uint16_t flags)
+{
+    return flags & nsHostResolver::RES_PRIORITY_LOW;
+}
+
+//----------------------------------------------------------------------------
+
 // this macro filters out any flags that are not used when constructing the
 // host key.  the significant flags are those that would affect the resulting
 // host record (i.e., the flags that are passed down to PR_GetAddrInfoByName).
@@ -151,19 +159,20 @@ private:
 nsHostRecord::nsHostRecord(const nsHostKey *key)
     : addr_info_lock("nsHostRecord.addr_info_lock")
     , addr_info_gencnt(0)
-    , addr_info(nsnull)
-    , addr(nsnull)
+    , addr_info(nullptr)
+    , addr(nullptr)
     , negative(false)
     , resolving(false)
     , onQueue(false)
     , usingAnyThread(false)
+    , mDoomed(false)
 {
     host = ((char *) this) + sizeof(nsHostRecord);
     memcpy((char *) host, key->host, strlen(key->host) + 1);
     flags = key->flags;
     af = key->af;
 
-    expiration = NowInMinutes();
+    expiration = TimeStamp::NowLoRes();
 
     PR_INIT_CLIST(this);
     PR_INIT_CLIST(&callbacks);
@@ -180,50 +189,57 @@ nsHostRecord::Create(const nsHostKey *key, nsHostRecord **result)
     void *place = ::operator new(size);
     *result = new(place) nsHostRecord(key);
     NS_ADDREF(*result);
+
+    MOZ_EVENT_TRACER_NAME_OBJECT(*result, key->host);
+
     return NS_OK;
 }
 
 nsHostRecord::~nsHostRecord()
 {
-    if (addr)
-        free(addr);
+    delete addr_info;
+    delete addr;
 }
 
 bool
-nsHostRecord::Blacklisted(PRNetAddr *aQuery)
+nsHostRecord::Blacklisted(NetAddr *aQuery)
 {
     // must call locked
-    LOG(("nsHostRecord::Blacklisted() %p %s\n", this, host));
+    LOG(("Checking blacklist for host [%s], host record [%p].\n", host, this));
 
     // skip the string conversion for the common case of no blacklist
-    if (!mBlacklistedItems.Length())
-        return false;
-    
-    char buf[64];
-    if (PR_NetAddrToString(aQuery, buf, sizeof(buf)) != PR_SUCCESS)
+    if (!mBlacklistedItems.Length()) {
         return false;
+    }
 
+    char buf[kIPv6CStrBufSize];
+    if (!NetAddrToString(aQuery, buf, sizeof(buf))) {
+        return false;
+    }
     nsDependentCString strQuery(buf);
-    LOG(("nsHostRecord::Blacklisted() query %s\n", buf));
-    
-    for (PRUint32 i = 0; i < mBlacklistedItems.Length(); i++)
+
+    for (uint32_t i = 0; i < mBlacklistedItems.Length(); i++) {
         if (mBlacklistedItems.ElementAt(i).Equals(strQuery)) {
-            LOG(("nsHostRecord::Blacklisted() %s blacklist confirmed\n", buf));
+            LOG(("Address [%s] is blacklisted for host [%s].\n", buf, host));
             return true;
         }
+    }
 
     return false;
 }
 
 void
-nsHostRecord::ReportUnusable(PRNetAddr *aAddress)
+nsHostRecord::ReportUnusable(NetAddr *aAddress)
 {
     // must call locked
-    LOG(("nsHostRecord::ReportUnusable() %p %s\n", this, host));
+    LOG(("Adding address to blacklist for host [%s], host record [%p].\n", host, this));
 
-    char buf[64];
-    if (PR_NetAddrToString(aAddress, buf, sizeof(buf)) == PR_SUCCESS) {
-        LOG(("nsHostrecord::ReportUnusable addr %s\n",buf));
+    if (negative)
+        mDoomed = true;
+
+    char buf[kIPv6CStrBufSize];
+    if (NetAddrToString(aAddress, buf, sizeof(buf))) {
+        LOG(("Successfully adding address [%s] to blacklist for host [%s].\n", buf, host));
         mBlacklistedItems.AppendElement(nsCString(buf));
     }
 }
@@ -232,10 +248,59 @@ void
 nsHostRecord::ResetBlacklist()
 {
     // must call locked
-    LOG(("nsHostRecord::ResetBlacklist() %p %s\n", this, host));
+    LOG(("Resetting blacklist for host [%s], host record [%p].\n", host, this));
     mBlacklistedItems.Clear();
 }
 
+bool
+nsHostRecord::HasUsableResult(uint16_t queryFlags) const
+{
+    if (mDoomed)
+        return false;
+
+    // don't use cached negative results for high priority queries.
+    if (negative && IsHighPriority(queryFlags))
+        return false;
+
+    return addr_info || addr || negative;
+}
+
+static size_t
+SizeOfResolveHostCallbackListExcludingHead(const PRCList *head,
+                                           MallocSizeOf mallocSizeOf)
+{
+    size_t n = 0;
+    PRCList *curr = head->next;
+    while (curr != head) {
+        nsResolveHostCallback *callback =
+            static_cast<nsResolveHostCallback*>(curr);
+        n += callback->SizeOfIncludingThis(mallocSizeOf);
+        curr = curr->next;
+    }
+    return n;
+}
+
+size_t
+nsHostRecord::SizeOfIncludingThis(MallocSizeOf mallocSizeOf) const
+{
+    size_t n = mallocSizeOf(this);
+
+    // The |host| field (inherited from nsHostKey) actually points to extra
+    // memory that is allocated beyond the end of the nsHostRecord (see
+    // nsHostRecord::Create()).  So it will be included in the
+    // |mallocSizeOf(this)| call above.
+
+    n += SizeOfResolveHostCallbackListExcludingHead(&callbacks, mallocSizeOf);
+    n += addr_info ? addr_info->SizeOfIncludingThis(mallocSizeOf) : 0;
+    n += mallocSizeOf(addr);
+
+    n += mBlacklistedItems.SizeOfExcludingThis(mallocSizeOf);
+    for (size_t i = 0; i < mBlacklistedItems.Length(); i++) {
+        n += mBlacklistedItems[i].SizeOfExcludingThisMustBeUnshared(mallocSizeOf);
+    }
+    return n;
+}
+
 //----------------------------------------------------------------------------
 
 struct nsHostDBEnt : PLDHashEntryHdr
@@ -276,26 +341,37 @@ static void
 HostDB_ClearEntry(PLDHashTable *table,
                   PLDHashEntryHdr *entry)
 {
-    LOG(("evicting record\n"));
-    nsHostDBEnt *he = static_cast<nsHostDBEnt *>(entry);
+    nsHostDBEnt *he = static_cast<nsHostDBEnt*>(entry);
+    MOZ_ASSERT(he, "nsHostDBEnt is null!");
+
+    nsHostRecord *hr = he->rec;
+    MOZ_ASSERT(hr, "nsHostDBEnt has null host record!");
+
+    LOG(("Clearing cache db entry for host [%s].\n", hr->host));
 #if defined(DEBUG) && defined(PR_LOGGING)
-    if (!he->rec->addr_info)
-        LOG(("%s: => no addr_info\n", he->rec->host));
-    else {
-        PRInt32 now = (PRInt32) NowInMinutes();
-        PRInt32 diff = (PRInt32) he->rec->expiration - now;
-        LOG(("%s: exp=%d => %s\n",
-            he->rec->host, diff,
-            PR_GetCanonNameFromAddrInfo(he->rec->addr_info)));
-        void *iter = nsnull;
-        PRNetAddr addr;
-        char buf[64];
-        for (;;) {
-            iter = PR_EnumerateAddrInfo(iter, he->rec->addr_info, 0, &addr);
-            if (!iter)
-                break;
-            PR_NetAddrToString(&addr, buf, sizeof(buf));
-            LOG(("  %s\n", buf));
+    {
+        MutexAutoLock lock(hr->addr_info_lock);
+        if (!hr->addr_info) {
+            LOG(("No address info for host [%s].\n", hr->host));
+        } else {
+            TimeDuration diff = hr->expiration - TimeStamp::NowLoRes();
+            LOG(("Record for [%s] expires in %f seconds.\n", hr->host, diff.ToSeconds()));
+
+            NetAddrElement *addrElement = nullptr;
+            char buf[kIPv6CStrBufSize];
+            do {
+                if (!addrElement) {
+                    addrElement = hr->addr_info->mAddresses.getFirst();
+                } else {
+                    addrElement = addrElement->getNext();
+                }
+
+                if (addrElement) {
+                    NetAddrToString(&addrElement->mAddress, buf, sizeof(buf));
+                    LOG(("  [%s]\n", buf));
+                }
+            }
+            while (addrElement);
         }
     }
 #endif
@@ -312,7 +388,7 @@ HostDB_InitEntry(PLDHashTable *table,
     return true;
 }
 
-static PLDHashTableOps gHostDB_ops =
+static const PLDHashTableOps gHostDB_ops =
 {
     PL_DHashAllocTable,
     PL_DHashFreeTable,
@@ -327,7 +403,7 @@ static PLDHashTableOps gHostDB_ops =
 static PLDHashOperator
 HostDB_RemoveEntry(PLDHashTable *table,
                    PLDHashEntryHdr *hdr,
-                   PRUint32 number,
+                   uint32_t number,
                    void *arg)
 {
     return PL_DHASH_REMOVE;
@@ -335,11 +411,11 @@ HostDB_RemoveEntry(PLDHashTable *table,
 
 //----------------------------------------------------------------------------
 
-nsHostResolver::nsHostResolver(PRUint32 maxCacheEntries,
-                               PRUint32 maxCacheLifetime,
-                               PRUint32 lifetimeGracePeriod)
+nsHostResolver::nsHostResolver(uint32_t maxCacheEntries,
+                               uint32_t maxCacheLifetime,
+                               uint32_t lifetimeGracePeriod)
     : mMaxCacheEntries(maxCacheEntries)
-    , mMaxCacheLifetime(maxCacheLifetime)
+    , mMaxCacheLifetime(TimeDuration::FromSeconds(maxCacheLifetime * 60))
     , mGracePeriod(lifetimeGracePeriod)
     , mLock("nsHostResolver.mLock")
     , mIdleThreadCV(mLock, "nsHostResolver.mIdleThreadCV")
@@ -368,9 +444,7 @@ nsHostResolver::~nsHostResolver()
 nsresult
 nsHostResolver::Init()
 {
-    NS_TIME_FUNCTION;
-
-    PL_DHashTableInit(&mDB, &gHostDB_ops, nsnull, sizeof(nsHostDBEnt), 0);
+    PL_DHashTableInit(&mDB, &gHostDB_ops, nullptr, sizeof(nsHostDBEnt), 0);
 
     mShutdown = false;
 
@@ -382,7 +456,7 @@ nsHostResolver::Init()
     // during application startup.
     static int initCount = 0;
     if (initCount++ > 0) {
-        LOG(("calling res_ninit\n"));
+        LOG(("Calling 'res_ninit'.\n"));
         res_ninit(&_res);
     }
 #endif
@@ -398,7 +472,7 @@ nsHostResolver::ClearPendingQueue(PRCList *aPendingQ)
         while (node != aPendingQ) {
             nsHostRecord *rec = static_cast<nsHostRecord *>(node);
             node = node->next;
-            OnLookupComplete(rec, NS_ERROR_ABORT, nsnull);
+            OnLookupComplete(rec, NS_ERROR_ABORT, nullptr);
         }
     }
 }
@@ -406,7 +480,7 @@ nsHostResolver::ClearPendingQueue(PRCList *aPendingQ)
 void
 nsHostResolver::Shutdown()
 {
-    LOG(("nsHostResolver::Shutdown\n"));
+    LOG(("Shutting down host resolver.\n"));
 
     PRCList pendingQHigh, pendingQMed, pendingQLow, evictionQ;
     PR_INIT_CLIST(&pendingQHigh);
@@ -430,7 +504,7 @@ nsHostResolver::Shutdown()
             mIdleThreadCV.NotifyAll();
         
         // empty host database
-        PL_DHashTableEnumerate(&mDB, HostDB_RemoveEntry, nsnull);
+        PL_DHashTableEnumerate(&mDB, HostDB_RemoveEntry, nullptr);
     }
     
     ClearPendingQueue(&pendingQHigh);
@@ -462,24 +536,6 @@ nsHostResolver::Shutdown()
 #endif
 }
 
-static inline bool
-IsHighPriority(PRUint16 flags)
-{
-    return !(flags & (nsHostResolver::RES_PRIORITY_LOW | nsHostResolver::RES_PRIORITY_MEDIUM));
-}
-
-static inline bool
-IsMediumPriority(PRUint16 flags)
-{
-    return flags & nsHostResolver::RES_PRIORITY_MEDIUM;
-}
-
-static inline bool
-IsLowPriority(PRUint16 flags)
-{
-    return flags & nsHostResolver::RES_PRIORITY_LOW;
-}
-
 void 
 nsHostResolver::MoveQueue(nsHostRecord *aRec, PRCList &aDestQ)
 {
@@ -491,13 +547,14 @@ nsHostResolver::MoveQueue(nsHostRecord *aRec, PRCList &aDestQ)
 
 nsresult
 nsHostResolver::ResolveHost(const char            *host,
-                            PRUint16               flags,
-                            PRUint16               af,
+                            uint16_t               flags,
+                            uint16_t               af,
                             nsResolveHostCallback *callback)
 {
     NS_ENSURE_TRUE(host && *host, NS_ERROR_UNEXPECTED);
 
-    LOG(("nsHostResolver::ResolveHost [host=%s]\n", host));
+    LOG(("Resolving host [%s]%s.\n",
+         host, flags & RES_BYPASS_CACHE ? " - bypassing cache" : ""));
 
     // ensure that we are working with a valid hostname before proceeding.  see
     // bug 304904 for details.
@@ -514,10 +571,10 @@ nsHostResolver::ResolveHost(const char            *host,
         if (mShutdown)
             rv = NS_ERROR_NOT_INITIALIZED;
         else {
+            // Used to try to parse to an IP address literal.
             PRNetAddr tempAddr;
-
-            // unfortunately, PR_StringToNetAddr does not properly initialize
-            // the output buffer in the case of IPv6 input.  see bug 223145.
+            // Unfortunately, PR_StringToNetAddr does not properly initialize
+            // the output buffer in the case of IPv6 input. See bug 223145.
             memset(&tempAddr, 0, sizeof(PRNetAddr));
             
             // check to see if there is already an entry for this |host|
@@ -532,36 +589,26 @@ nsHostResolver::ResolveHost(const char            *host,
                                          (PL_DHashTableOperate(&mDB, &key, PL_DHASH_ADD));
 
             // if the record is null, then HostDB_InitEntry failed.
-            if (!he || !he->rec)
+            if (!he || !he->rec) {
+                LOG(("  Out of memory: no cache entry for [%s].\n", host));
                 rv = NS_ERROR_OUT_OF_MEMORY;
+            }
             // do we have a cached result that we can reuse?
             else if (!(flags & RES_BYPASS_CACHE) &&
-                     he->rec->HasResult() &&
-                     NowInMinutes() <= he->rec->expiration + mGracePeriod) {
-                        
-                LOG(("using cached record\n"));
+                     he->rec->HasUsableResult(flags) &&
+                     TimeStamp::NowLoRes() <= (he->rec->expiration + TimeDuration::FromSeconds(mGracePeriod * 60))) {
+                LOG(("  Using cached record for host [%s].\n", host));
                 // put reference to host record on stack...
                 result = he->rec;
                 Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2, METHOD_HIT);
 
-                // For entries that are in the grace period, or all cached
-                // negative entries, use the cache but start a new lookup in
-                // the background
-                if (((NowInMinutes() > he->rec->expiration) ||
-                     he->rec->negative) && !he->rec->resolving) {
-                    LOG(("Using %s cache entry but starting async renewal",
-                         he->rec->negative ? "negative" :"positive"));
-                    IssueLookup(he->rec);
-
-                    if (!he->rec->negative) {
-                        // negative entries are constantly being refreshed, only
-                        // track positive grace period induced renewals
-                        Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
-                                              METHOD_RENEWAL);
-                    }
-                }
+                // For entries that are in the grace period
+                // or all cached negative entries, use the cache but start a new
+                // lookup in the background
+                ConditionallyRefreshRecord(he->rec, host);
                 
                 if (he->rec->negative) {
+                    LOG(("  Negative cache entry for[%s].\n", host));
                     Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                           METHOD_NEGATIVE_HIT);
                     status = NS_ERROR_UNKNOWN_HOST;
@@ -570,6 +617,7 @@ nsHostResolver::ResolveHost(const char            *host,
             // if the host name is an IP address literal and has been parsed,
             // go ahead and use it.
             else if (he->rec->addr) {
+                LOG(("  Using cached address for IP Literal [%s].\n", host));
                 Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                       METHOD_LITERAL);
                 result = he->rec;
@@ -578,13 +626,11 @@ nsHostResolver::ResolveHost(const char            *host,
             // circuit full host resolution.  (this is necessary on some
             // platforms like Win9x.  see bug 219376 for more details.)
             else if (PR_StringToNetAddr(host, &tempAddr) == PR_SUCCESS) {
+                LOG(("  Host is IP Literal [%s].\n", host));
                 // ok, just copy the result into the host record, and be done
                 // with it! ;-)
-                he->rec->addr = (PRNetAddr *) malloc(sizeof(PRNetAddr));
-                if (!he->rec->addr)
-                    status = NS_ERROR_OUT_OF_MEMORY;
-                else
-                    memcpy(he->rec->addr, &tempAddr, sizeof(PRNetAddr));
+                he->rec->addr = new NetAddr();
+                PRNetAddrToNetAddr(&tempAddr, he->rec->addr);
                 // put reference to host record on stack...
                 Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                       METHOD_LITERAL);
@@ -593,40 +639,128 @@ nsHostResolver::ResolveHost(const char            *host,
             else if (mPendingCount >= MAX_NON_PRIORITY_REQUESTS &&
                      !IsHighPriority(flags) &&
                      !he->rec->resolving) {
+                LOG(("  Lookup queue full: dropping %s priority request for "
+                     "[%s].\n",
+                     IsMediumPriority(flags) ? "medium" : "low", host));
                 Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                       METHOD_OVERFLOW);
                 // This is a lower priority request and we are swamped, so refuse it.
                 rv = NS_ERROR_DNS_LOOKUP_QUEUE_FULL;
             }
-            // otherwise, hit the resolver...
-            else {
-                // Add callback to the list of pending callbacks.
-                PR_APPEND_LINK(callback, &he->rec->callbacks);
+            else if (flags & RES_OFFLINE) {
+                LOG(("  Offline request for [%s]; ignoring.\n", host));
+                rv = NS_ERROR_OFFLINE;
+            }
 
-                if (!he->rec->resolving) {
+            // If this is an IPV4 or IPV6 specific request, check if there is
+            // an AF_UNSPEC entry we can use. Otherwise, hit the resolver...
+            else if (!he->rec->resolving) {
+                if (!(flags & RES_BYPASS_CACHE) &&
+                    ((af == PR_AF_INET) || (af == PR_AF_INET6))) {
+                    // First, search for an entry with AF_UNSPEC
+                    const nsHostKey unspecKey = { host, flags, PR_AF_UNSPEC };
+                    nsHostDBEnt *unspecHe = static_cast<nsHostDBEnt *>
+                        (PL_DHashTableOperate(&mDB, &unspecKey, PL_DHASH_LOOKUP));
+                    NS_ASSERTION(PL_DHASH_ENTRY_IS_FREE(unspecHe) ||
+                                 (PL_DHASH_ENTRY_IS_BUSY(unspecHe) &&
+                                  unspecHe->rec),
+                                "Valid host entries should contain a record");
+                    if (PL_DHASH_ENTRY_IS_BUSY(unspecHe) &&
+                        unspecHe->rec &&
+                        unspecHe->rec->HasUsableResult(flags) &&
+                        TimeStamp::NowLoRes() <= (he->rec->expiration +
+                            TimeDuration::FromSeconds(mGracePeriod * 60))) {
+
+                        MOZ_ASSERT(unspecHe->rec->addr_info || unspecHe->rec->negative,
+                                   "Entry should be resolved or negative.");
+
+                        LOG(("  Trying AF_UNSPEC entry for [%s] af: %s.\n",
+                            host, (af == PR_AF_INET) ? "AF_INET" : "AF_INET6"));
+
+                        he->rec->addr_info = nullptr;
+                        if (unspecHe->rec->negative) {
+                            he->rec->negative = unspecHe->rec->negative;
+                        } else if (unspecHe->rec->addr_info) {
+                            // Search for any valid address in the AF_UNSPEC entry
+                            // in the cache (not blacklisted and from the right
+                            // family).
+                            NetAddrElement *addrIter =
+                                unspecHe->rec->addr_info->mAddresses.getFirst();
+                            while (addrIter) {
+                                if ((af == addrIter->mAddress.inet.family) &&
+                                     !unspecHe->rec->Blacklisted(&addrIter->mAddress)) {
+                                    if (!he->rec->addr_info) {
+                                        he->rec->addr_info = new AddrInfo(
+                                            unspecHe->rec->addr_info->mHostName,
+                                            unspecHe->rec->addr_info->mCanonicalName);
+                                    }
+                                    he->rec->addr_info->AddAddress(
+                                        new NetAddrElement(*addrIter));
+                                }
+                                addrIter = addrIter->getNext();
+                            }
+                        }
+                        if (he->rec->HasUsableResult(flags)) {
+                            result = he->rec;
+                            Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
+                                                  METHOD_HIT);
+                            ConditionallyRefreshRecord(he->rec, host);
+                        }
+                        // For AF_INET6, a new lookup means another AF_UNSPEC
+                        // lookup. We have already iterated through the
+                        // AF_UNSPEC addresses, so we mark this record as
+                        // negative.
+                        else if (af == PR_AF_INET6) {
+                            LOG(("  No AF_INET6 in AF_UNSPEC entry: "
+                                 "[%s] unknown host", host));
+                            result = he->rec;
+                            he->rec->negative = true;
+                            status = NS_ERROR_UNKNOWN_HOST;
+                            Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
+                                                  METHOD_NEGATIVE_HIT);
+                        }
+                    }
+                }
+                // If no valid address was found in the cache or this is an
+                // AF_UNSPEC request, then start a new lookup.
+                if (!result) {
+                    LOG(("  No usable address in cache for [%s]", host));
+                    // Add callback to the list of pending callbacks.
+                    PR_APPEND_LINK(callback, &he->rec->callbacks);
                     he->rec->flags = flags;
                     rv = IssueLookup(he->rec);
                     Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                           METHOD_NETWORK_FIRST);
-                    if (NS_FAILED(rv))
+                    if (NS_FAILED(rv)) {
                         PR_REMOVE_AND_INIT_LINK(callback);
-                    else
-                        LOG(("dns lookup blocking pending getaddrinfo query"));
+                    }
+                    else {
+                        LOG(("  DNS lookup for host [%s] blocking pending "
+                             "'getaddrinfo' query: callback [%p]",
+                             host, callback));
+                    }
                 }
-                else if (he->rec->onQueue) {
+            }
+            else {
+                LOG(("  Host [%s] is being resolved. Appending callback [%p].",
+                     host, callback));
+                PR_APPEND_LINK(callback, &he->rec->callbacks);
+                if (he->rec->onQueue) {
                     Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
                                           METHOD_NETWORK_SHARED);
 
-                    // Consider the case where we are on a pending queue of 
+                    // Consider the case where we are on a pending queue of
                     // lower priority than the request is being made at.
                     // In that case we should upgrade to the higher queue.
 
-                    if (IsHighPriority(flags) && !IsHighPriority(he->rec->flags)) {
+                    if (IsHighPriority(flags) &&
+                        !IsHighPriority(he->rec->flags)) {
                         // Move from (low|med) to high.
                         MoveQueue(he->rec, mHighQ);
                         he->rec->flags = flags;
                         ConditionallyCreateThread(he->rec);
-                    } else if (IsMediumPriority(flags) && IsLowPriority(he->rec->flags)) {
+                    } else if (IsMediumPriority(flags) &&
+                               IsLowPriority(he->rec->flags)) {
                         // Move from low to med.
                         MoveQueue(he->rec, mMediumQ);
                         he->rec->flags = flags;
@@ -643,8 +777,8 @@ nsHostResolver::ResolveHost(const char            *host,
 
 void
 nsHostResolver::DetachCallback(const char            *host,
-                               PRUint16               flags,
-                               PRUint16               af,
+                               uint16_t               flags,
+                               uint16_t               af,
                                nsResolveHostCallback *callback,
                                nsresult               status)
 {
@@ -704,7 +838,7 @@ nsHostResolver::ConditionallyCreateThread(nsHostRecord *rec)
     }
 #if defined(PR_LOGGING)
     else
-      LOG(("lookup waiting for thread - %s ...\n", rec->host));
+      LOG(("  Unable to find a thread for looking up host [%s].\n", rec->host));
 #endif
     return NS_OK;
 }
@@ -712,8 +846,10 @@ nsHostResolver::ConditionallyCreateThread(nsHostRecord *rec)
 nsresult
 nsHostResolver::IssueLookup(nsHostRecord *rec)
 {
+    MOZ_EVENT_TRACER_WAIT(rec, "net::dns::resolve");
+
     nsresult rv = NS_OK;
-    NS_ASSERTION(!rec->resolving, "record is already being resolved"); 
+    NS_ASSERTION(!rec->resolving, "record is already being resolved");
 
     // Add rec to one of the pending queues, possibly removing it from mEvictionQ.
     // If rec is on mEvictionQ, then we can just move the owning
@@ -738,7 +874,7 @@ nsHostResolver::IssueLookup(nsHostRecord *rec)
 
     rv = ConditionallyCreateThread(rec);
     
-    LOG (("DNS Thread Counters: total=%d any-live=%d idle=%d pending=%d\n",
+    LOG (("  DNS thread counters: total=%d any-live=%d idle=%d pending=%d\n",
           mThreadCount,
           mActiveAnyThreadCount,
           mNumIdleThreads,
@@ -747,6 +883,25 @@ nsHostResolver::IssueLookup(nsHostRecord *rec)
     return rv;
 }
 
+nsresult
+nsHostResolver::ConditionallyRefreshRecord(nsHostRecord *rec, const char *host)
+{
+    if (((TimeStamp::NowLoRes() > rec->expiration) || rec->negative) &&
+        !rec->resolving) {
+        LOG(("  Using %s cache entry for host [%s] but starting async renewal.",
+            rec->negative ? "negative" :"positive", host));
+        IssueLookup(rec);
+
+        if (!rec->negative) {
+            // negative entries are constantly being refreshed, only
+            // track positive grace period induced renewals
+            Telemetry::Accumulate(Telemetry::DNS_LOOKUP_METHOD2,
+                METHOD_RENEWAL);
+        }
+    }
+    return NS_OK;
+}
+
 void
 nsHostResolver::DeQueue(PRCList &aQ, nsHostRecord **aResult)
 {
@@ -824,7 +979,7 @@ nsHostResolver::GetHostToLookup(nsHostRecord **result)
 }
 
 void
-nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo *result)
+nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, AddrInfo *result)
 {
     // get the list of pending callbacks for this lookup, and notify
     // them that the lookup is complete.
@@ -838,22 +993,22 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
 
         // update record fields.  We might have a rec->addr_info already if a
         // previous lookup result expired and we're reresolving it..
-        PRAddrInfo  *old_addr_info;
+        AddrInfo  *old_addr_info;
         {
             MutexAutoLock lock(rec->addr_info_lock);
             old_addr_info = rec->addr_info;
             rec->addr_info = result;
             rec->addr_info_gencnt++;
         }
-        if (old_addr_info)
-            PR_FreeAddrInfo(old_addr_info);
-        rec->expiration = NowInMinutes();
+        delete old_addr_info;
+
+        rec->expiration = TimeStamp::NowLoRes();
         if (result) {
             rec->expiration += mMaxCacheLifetime;
             rec->negative = false;
         }
         else {
-            rec->expiration += 1;                 /* one minute for negative cache */
+            rec->expiration += TimeDuration::FromSeconds(60); /* one minute for negative cache */
             rec->negative = true;
         }
         rec->resolving = false;
@@ -863,7 +1018,7 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
             rec->usingAnyThread = false;
         }
 
-        if (rec->addr_info && !mShutdown) {
+        if (!mShutdown) {
             // add to mEvictionQ
             PR_APPEND_LINK(rec, &mEvictionQ);
             NS_ADDREF(rec);
@@ -878,9 +1033,10 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
 
                 if (!head->negative) {
                     // record the age of the entry upon eviction.
-                    PRUint32 age =
-                        NowInMinutes() - (head->expiration - mMaxCacheLifetime);
-                    Telemetry::Accumulate(Telemetry::DNS_CLEANUP_AGE, age);
+                    TimeDuration age = TimeStamp::NowLoRes() -
+                                         (head->expiration - mMaxCacheLifetime);
+                    Telemetry::Accumulate(Telemetry::DNS_CLEANUP_AGE,
+                                          static_cast<uint32_t>(age.ToSeconds() / 60));
                 }
 
                 // release reference to rec owned by mEvictionQ
@@ -889,6 +1045,8 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
         }
     }
 
+    MOZ_EVENT_TRACER_DONE(rec, "net::dns::resolve");
+
     if (!PR_CLIST_IS_EMPTY(&cbs)) {
         PRCList *node = cbs.next;
         while (node != &cbs) {
@@ -905,8 +1063,8 @@ nsHostResolver::OnLookupComplete(nsHostRecord *rec, nsresult status, PRAddrInfo
 
 void
 nsHostResolver::CancelAsyncRequest(const char            *host,
-                                   PRUint16               flags,
-                                   PRUint16               af,
+                                   uint16_t               flags,
+                                   uint16_t               af,
                                    nsIDNSListener        *aListener,
                                    nsresult               status)
 
@@ -918,7 +1076,7 @@ nsHostResolver::CancelAsyncRequest(const char            *host,
     nsHostDBEnt *he = static_cast<nsHostDBEnt *>
                       (PL_DHashTableOperate(&mDB, &key, PL_DHASH_LOOKUP));
     if (he && he->rec) {
-        nsHostRecord* recPtr = NULL;
+        nsHostRecord* recPtr = nullptr;
         PRCList *node = he->rec->callbacks.next;
         // Remove the first nsDNSAsyncRequest callback which matches the
         // supplied listener object
@@ -947,12 +1105,35 @@ nsHostResolver::CancelAsyncRequest(const char            *host,
     }
 }
 
-//----------------------------------------------------------------------------
+static size_t
+SizeOfHostDBEntExcludingThis(PLDHashEntryHdr* hdr, MallocSizeOf mallocSizeOf,
+                             void*)
+{
+    nsHostDBEnt* ent = static_cast<nsHostDBEnt*>(hdr);
+    return ent->rec->SizeOfIncludingThis(mallocSizeOf);
+}
+
+size_t
+nsHostResolver::SizeOfIncludingThis(MallocSizeOf mallocSizeOf) const
+{
+    MutexAutoLock lock(mLock);
+
+    size_t n = mallocSizeOf(this);
+    n += PL_DHashTableSizeOfExcludingThis(&mDB, SizeOfHostDBEntExcludingThis,
+                                          mallocSizeOf);
+
+    // The following fields aren't measured.
+    // - mHighQ, mMediumQ, mLowQ, mEvictionQ, because they just point to
+    //   nsHostRecords that also pointed to by entries |mDB|, and measured when
+    //   |mDB| is measured.
+
+    return n;
+}
 
 void
 nsHostResolver::ThreadFunc(void *arg)
 {
-    LOG(("nsHostResolver::ThreadFunc entering\n"));
+    LOG(("DNS lookup thread - starting execution.\n"));
 
     static nsThreadPoolNaming naming;
     naming.SetThreadPoolName(NS_LITERAL_CSTRING("DNS Resolver"));
@@ -962,27 +1143,45 @@ nsHostResolver::ThreadFunc(void *arg)
 #endif
     nsHostResolver *resolver = (nsHostResolver *)arg;
     nsHostRecord *rec;
-    PRAddrInfo *ai;
+    PRAddrInfo *prai = nullptr;
     while (resolver->GetHostToLookup(&rec)) {
-        LOG(("resolving %s ...\n", rec->host));
+        LOG(("DNS lookup thread - Calling getaddrinfo for host [%s].\n",
+             rec->host));
 
-        PRIntn flags = PR_AI_ADDRCONFIG;
+        int flags = PR_AI_ADDRCONFIG;
         if (!(rec->flags & RES_CANON_NAME))
             flags |= PR_AI_NOCANONNAME;
 
         TimeStamp startTime = TimeStamp::Now();
+        MOZ_EVENT_TRACER_EXEC(rec, "net::dns::resolve");
 
-        ai = PR_GetAddrInfoByName(rec->host, rec->af, flags);
+        // We need to remove IPv4 records manually
+        // because PR_GetAddrInfoByName doesn't support PR_AF_INET6.
+        bool disableIPv4 = rec->af == PR_AF_INET6;
+        uint16_t af = disableIPv4 ? PR_AF_UNSPEC : rec->af;
+        prai = PR_GetAddrInfoByName(rec->host, af, flags);
 #if defined(RES_RETRY_ON_FAILURE)
-        if (!ai && rs.Reset())
-            ai = PR_GetAddrInfoByName(rec->host, rec->af, flags);
+        if (!prai && rs.Reset())
+            prai = PR_GetAddrInfoByName(rec->host, af, flags);
 #endif
 
         TimeDuration elapsed = TimeStamp::Now() - startTime;
-        PRUint32 millis = static_cast<PRUint32>(elapsed.ToMilliseconds());
+        uint32_t millis = static_cast<uint32_t>(elapsed.ToMilliseconds());
 
-        // convert error code to nsresult.
+        // convert error code to nsresult
         nsresult status;
+        AddrInfo *ai = nullptr;
+        if (prai) {
+            const char *cname = nullptr;
+            if (rec->flags & RES_CANON_NAME)
+                cname = PR_GetCanonNameFromAddrInfo(prai);
+            ai = new AddrInfo(rec->host, prai, disableIPv4, cname);
+            PR_FreeAddrInfo(prai);
+            if (ai->mAddresses.isEmpty()) {
+                delete ai;
+                ai = nullptr;
+            }
+        }
         if (ai) {
             status = NS_OK;
 
@@ -995,20 +1194,20 @@ nsHostResolver::ThreadFunc(void *arg)
             status = NS_ERROR_UNKNOWN_HOST;
             Telemetry::Accumulate(Telemetry::DNS_FAILED_LOOKUP_TIME, millis);
         }
-        
+
+        // OnLookupComplete may release "rec", log before we lose it.
+        LOG(("DNS lookup thread - lookup completed for host [%s]: %s.\n",
+             rec->host, ai ? "success" : "failure: unknown host"));
         resolver->OnLookupComplete(rec, status, ai);
-        LOG(("lookup complete for %s ...\n", rec->host));
     }
     NS_RELEASE(resolver);
-    LOG(("nsHostResolver::ThreadFunc exiting\n"));
+    LOG(("DNS lookup thread - queue empty, thread finished.\n"));
 }
 
-//----------------------------------------------------------------------------
-
 nsresult
-nsHostResolver::Create(PRUint32         maxCacheEntries,
-                       PRUint32         maxCacheLifetime,
-                       PRUint32         lifetimeGracePeriod,
+nsHostResolver::Create(uint32_t         maxCacheEntries,
+                       uint32_t         maxCacheLifetime,
+                       uint32_t         lifetimeGracePeriod,
                        nsHostResolver **result)
 {
 #if defined(PR_LOGGING)
@@ -1030,3 +1229,57 @@ nsHostResolver::Create(PRUint32         maxCacheEntries,
     *result = res;
     return rv;
 }
+
+PLDHashOperator
+CacheEntryEnumerator(PLDHashTable *table, PLDHashEntryHdr *entry,
+                     uint32_t number, void *arg)
+{
+    // We don't pay attention to address literals, only resolved domains.
+    // Also require a host.
+    nsHostRecord *rec = static_cast<nsHostDBEnt*>(entry)->rec;
+    MOZ_ASSERT(rec, "rec should never be null here!");
+    if (!rec || !rec->addr_info || !rec->host) {
+        return PL_DHASH_NEXT;
+    }
+
+    DNSCacheEntries info;
+    info.hostname = rec->host;
+    info.family = rec->af;
+    info.expiration = (int64_t)(rec->expiration - TimeStamp::NowLoRes()).ToSeconds();
+    if (info.expiration <= 0) {
+        // We only need valid DNS cache entries
+        return PL_DHASH_NEXT;
+    }
+
+    {
+        MutexAutoLock lock(rec->addr_info_lock);
+
+        NetAddr *addr = nullptr;
+        NetAddrElement *addrElement = rec->addr_info->mAddresses.getFirst();
+        if (addrElement) {
+            addr = &addrElement->mAddress;
+        }
+        while (addr) {
+            char buf[kIPv6CStrBufSize];
+            if (NetAddrToString(addr, buf, sizeof(buf))) {
+                info.hostaddr.AppendElement(buf);
+            }
+            addr = nullptr;
+            addrElement = addrElement->getNext();
+            if (addrElement) {
+                addr = &addrElement->mAddress;
+            }
+        }
+    }
+
+    nsTArray<DNSCacheEntries> *args = static_cast<nsTArray<DNSCacheEntries> *>(arg);
+    args->AppendElement(info);
+
+    return PL_DHASH_NEXT;
+}
+
+void
+nsHostResolver::GetDNSCacheEntries(nsTArray<DNSCacheEntries> *args)
+{
+    PL_DHashTableEnumerate(&mDB, CacheEntryEnumerator, args);
+}
