@@ -3,8 +3,9 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+// IWYU pragma: private, include "nsString.h"
 
-// IWYU pragma: private, include "nsAString.h"
+#include "mozilla/MemoryReporting.h"
 
 #ifndef MOZILLA_INTERNAL_API
 #error Cannot use internal string classes without MOZILLA_INTERNAL_API defined. Use the frozen header nsStringAPI.h instead.
@@ -20,7 +21,7 @@ class nsTStringComparator_CharT
 
       nsTStringComparator_CharT() {}
 
-      virtual int operator()( const char_type*, const char_type*, PRUint32, PRUint32 ) const = 0;
+      virtual int operator()( const char_type*, const char_type*, uint32_t, uint32_t ) const = 0;
   };
 
 
@@ -35,7 +36,7 @@ class nsTDefaultStringComparator_CharT
 
       nsTDefaultStringComparator_CharT() {}
 
-      virtual int operator()( const char_type*, const char_type*, PRUint32, PRUint32 ) const;
+      virtual int operator()( const char_type*, const char_type*, uint32_t, uint32_t ) const;
   };
 
   /**
@@ -76,8 +77,8 @@ class nsTSubstring_CharT
       typedef char_type*                          char_iterator;
       typedef const char_type*                    const_char_iterator;
 
-      typedef PRUint32                            size_type;
-      typedef PRUint32                            index_type;
+      typedef uint32_t                            size_type;
+      typedef uint32_t                            index_type;
 
     public:
 
@@ -129,7 +130,7 @@ class nsTSubstring_CharT
       char_iterator BeginWriting()
         {
           if (!EnsureMutable())
-            NS_RUNTIMEABORT("OOM");
+            NS_ABORT_OOM(mLength);
 
           return mData;
         }
@@ -142,7 +143,7 @@ class nsTSubstring_CharT
       char_iterator EndWriting()
         {
           if (!EnsureMutable())
-            NS_RUNTIMEABORT("OOM");
+            NS_ABORT_OOM(mLength);
 
           return mData + mLength;
         }
@@ -199,7 +200,11 @@ class nsTSubstring_CharT
          */
 
         // returns pointer to string data (not necessarily null-terminated)
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      char16ptr_t Data() const
+#else
       const char_type *Data() const
+#endif
         {
           return mData;
         }
@@ -209,11 +214,21 @@ class nsTSubstring_CharT
           return mLength;
         }
 
+      uint32_t Flags() const
+        {
+          return mFlags;
+        }
+
       bool IsEmpty() const
         {
           return mLength == 0;
         }
 
+      bool IsLiteral() const
+        {
+          return (mFlags & F_LITERAL) != 0;
+        }
+
       bool IsVoid() const
         {
           return (mFlags & F_VOIDED) != 0;
@@ -249,7 +264,7 @@ class nsTSubstring_CharT
         }
 
       size_type NS_FASTCALL CountChar( char_type ) const;
-      PRInt32 NS_FASTCALL FindChar( char_type, index_type offset = 0 ) const;
+      int32_t NS_FASTCALL FindChar( char_type, index_type offset = 0 ) const;
 
 
         /**
@@ -262,6 +277,17 @@ class nsTSubstring_CharT
       bool NS_FASTCALL Equals( const char_type* data ) const;
       bool NS_FASTCALL Equals( const char_type* data, const comparator_type& comp ) const;
 
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      bool NS_FASTCALL Equals( char16ptr_t data ) const
+        {
+          return Equals(static_cast<const char16_t*>(data));
+        }
+      bool NS_FASTCALL Equals( char16ptr_t data, const comparator_type& comp ) const
+        {
+          return Equals(static_cast<const char16_t*>(data), comp);
+        }
+#endif
+
         /**
          * An efficient comparison with ASCII that can be used even
          * for wide strings. Call this version when you know the
@@ -275,60 +301,37 @@ class nsTSubstring_CharT
          */
       bool NS_FASTCALL EqualsASCII( const char* data ) const;
 
-    // EqualsLiteral must ONLY be applied to an actual literal string.
-    // Do not attempt to use it with a regular char* pointer, or with a char
-    // array variable.
+    // EqualsLiteral must ONLY be applied to an actual literal string, or
+    // a char array *constant* declared without an explicit size.
+    // Do not attempt to use it with a regular char* pointer, or with a
+    // non-constant char array variable. Use EqualsASCII for them.
     // The template trick to acquire the array length at compile time without
     // using a macro is due to Corey Kosak, with much thanks.
-#ifdef NS_DISABLE_LITERAL_TEMPLATE
-      inline bool EqualsLiteral( const char* str ) const
-        {
-          return EqualsASCII(str);
-        }
-#else
       template<int N>
       inline bool EqualsLiteral( const char (&str)[N] ) const
         {
           return EqualsASCII(str, N-1);
         }
-      template<int N>
-      inline bool EqualsLiteral( char (&str)[N] ) const
-        {
-          const char* s = str;
-          return EqualsASCII(s, N-1);
-        }
-#endif
 
-    // The LowerCaseEquals methods compare the lower case version of
-    // this string to some ASCII/Literal string. The ASCII string is
-    // *not* lowercased for you. If you compare to an ASCII or literal
-    // string that contains an uppercase character, it is guaranteed to
-    // return false. We will throw assertions too.
+    // The LowerCaseEquals methods compare the ASCII-lowercase version of
+    // this string (lowercasing only ASCII uppercase characters) to some
+    // ASCII/Literal string. The ASCII string is *not* lowercased for
+    // you. If you compare to an ASCII or literal string that contains an
+    // uppercase character, it is guaranteed to return false. We will
+    // throw assertions too.
       bool NS_FASTCALL LowerCaseEqualsASCII( const char* data, size_type len ) const;
       bool NS_FASTCALL LowerCaseEqualsASCII( const char* data ) const;
 
     // LowerCaseEqualsLiteral must ONLY be applied to an actual
-    // literal string.  Do not attempt to use it with a regular char*
-    // pointer, or with a char array variable. Use
+    // literal string, or a char array *constant* declared without an 
+    // explicit size.  Do not attempt to use it with a regular char*
+    // pointer, or with a non-constant char array variable. Use
     // LowerCaseEqualsASCII for them.
-#ifdef NS_DISABLE_LITERAL_TEMPLATE
-      inline bool LowerCaseEqualsLiteral( const char* str ) const
-        {
-          return LowerCaseEqualsASCII(str);
-        }
-#else
       template<int N>
       inline bool LowerCaseEqualsLiteral( const char (&str)[N] ) const
         {
           return LowerCaseEqualsASCII(str, N-1);
         }
-      template<int N>
-      inline bool LowerCaseEqualsLiteral( char (&str)[N] ) const
-        {
-          const char* s = str;
-          return LowerCaseEqualsASCII(s, N-1);
-        }
-#endif
 
         /**
          * assignment
@@ -337,8 +340,8 @@ class nsTSubstring_CharT
       void NS_FASTCALL Assign( char_type c );
       bool NS_FASTCALL Assign( char_type c, const fallible_t& ) NS_WARN_UNUSED_RESULT;
 
-      void NS_FASTCALL
-        Assign( const char_type* data, size_type length = size_type(-1) );
+      void NS_FASTCALL Assign( const char_type* data );
+      void NS_FASTCALL Assign( const char_type* data, size_type length );
       bool NS_FASTCALL Assign( const char_type* data, size_type length, const fallible_t& ) NS_WARN_UNUSED_RESULT;
 
       void NS_FASTCALL Assign( const self_type& );
@@ -347,6 +350,28 @@ class nsTSubstring_CharT
       void NS_FASTCALL Assign( const substring_tuple_type& );
       bool NS_FASTCALL Assign( const substring_tuple_type&, const fallible_t& ) NS_WARN_UNUSED_RESULT;
 
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      void Assign (char16ptr_t data)
+        {
+          Assign(static_cast<const char16_t*>(data));
+        }
+
+      bool Assign(char16ptr_t data, const fallible_t&) NS_WARN_UNUSED_RESULT
+        {
+          return Assign(static_cast<const char16_t*>(data), fallible_t());
+        }
+
+      void Assign (char16ptr_t data, size_type length)
+        {
+          Assign(static_cast<const char16_t*>(data), length);
+        }
+
+      bool Assign(char16ptr_t data, size_type length, const fallible_t&) NS_WARN_UNUSED_RESULT
+        {
+          return Assign(static_cast<const char16_t*>(data), length, fallible_t());
+        }
+#endif
+
       void NS_FASTCALL AssignASCII( const char* data, size_type length );
       bool NS_FASTCALL AssignASCII( const char* data, size_type length, const fallible_t& ) NS_WARN_UNUSED_RESULT;
 
@@ -359,25 +384,26 @@ class nsTSubstring_CharT
           return AssignASCII(data, strlen(data), fallible_t());
         }
 
-    // AssignLiteral must ONLY be applied to an actual literal string.
-    // Do not attempt to use it with a regular char* pointer, or with a char
-    // array variable. Use AssignASCII for those.
+    // AssignLiteral must ONLY be applied to an actual literal string, or
+    // a char array *constant* declared without an explicit size.
+    // Do not attempt to use it with a regular char* pointer, or with a 
+    // non-constant char array variable. Use AssignASCII for those.
     // There are not fallible version of these methods because they only really
     // apply to small allocations that we wouldn't want to check anyway.
-#ifdef NS_DISABLE_LITERAL_TEMPLATE
-      void AssignLiteral( const char* str )
-                  { AssignASCII(str); }
-#else
+      template<int N>
+      void AssignLiteral( const char_type (&str)[N] )
+                  { AssignLiteral(str, N - 1); }
+#ifdef CharT_is_PRUnichar
       template<int N>
       void AssignLiteral( const char (&str)[N] )
                   { AssignASCII(str, N-1); }
-      template<int N>
-      void AssignLiteral( char (&str)[N] )
-                  { AssignASCII(str, N-1); }
 #endif
 
       self_type& operator=( char_type c )                                                       { Assign(c);        return *this; }
       self_type& operator=( const char_type* data )                                             { Assign(data);     return *this; }
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      self_type& operator=( char16ptr_t data )                                                  { Assign(data);     return *this; }
+#endif
       self_type& operator=( const self_type& str )                                              { Assign(str);      return *this; }
       self_type& operator=( const substring_tuple_type& tuple )                                 { Assign(tuple);    return *this; }
 
@@ -395,8 +421,19 @@ class nsTSubstring_CharT
 
       void NS_FASTCALL ReplaceASCII( index_type cutStart, size_type cutLength, const char* data, size_type length = size_type(-1) );
 
+    // ReplaceLiteral must ONLY be applied to an actual literal string.
+    // Do not attempt to use it with a regular char* pointer, or with a char
+    // array variable. Use Replace or ReplaceASCII for those.
+      template<int N>
+      void ReplaceLiteral( index_type cutStart, size_type cutLength, const char_type (&str)[N] ) { ReplaceLiteral(cutStart, cutLength, str, N - 1); }
+
       void Append( char_type c )                                                                 { Replace(mLength, 0, c); }
       void Append( const char_type* data, size_type length = size_type(-1) )                     { Replace(mLength, 0, data, length); }
+
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+    void Append( char16ptr_t data, size_type length = size_type(-1) )                            { Append(static_cast<const char16_t*>(data), length); }
+#endif
+
       void Append( const self_type& str )                                                        { Replace(mLength, 0, str); }
       void Append( const substring_tuple_type& tuple )                                           { Replace(mLength, 0, tuple); }
 
@@ -407,30 +444,31 @@ class nsTSubstring_CharT
        * codes documented in prprf.h
        */
       void AppendPrintf( const char* format, ... );
-      void AppendInt( PRInt32 aInteger )
+      void AppendPrintf( const char* format, va_list ap );
+      void AppendInt( int32_t aInteger )
                  { AppendPrintf( "%d", aInteger ); }
-      void AppendInt( PRInt32 aInteger, int aRadix )
+      void AppendInt( int32_t aInteger, int aRadix )
         {
           const char *fmt = aRadix == 10 ? "%d" : aRadix == 8 ? "%o" : "%x";
           AppendPrintf( fmt, aInteger );
         }
-      void AppendInt( PRUint32 aInteger )
+      void AppendInt( uint32_t aInteger )
                  { AppendPrintf( "%u", aInteger ); }
-      void AppendInt( PRUint32 aInteger, int aRadix )
+      void AppendInt( uint32_t aInteger, int aRadix )
         {
           const char *fmt = aRadix == 10 ? "%u" : aRadix == 8 ? "%o" : "%x";
           AppendPrintf( fmt, aInteger );
         }
-      void AppendInt( PRInt64 aInteger )
+      void AppendInt( int64_t aInteger )
                  { AppendPrintf( "%lld", aInteger ); }
-      void AppendInt( PRInt64 aInteger, int aRadix )
+      void AppendInt( int64_t aInteger, int aRadix )
         {
           const char *fmt = aRadix == 10 ? "%lld" : aRadix == 8 ? "%llo" : "%llx";
           AppendPrintf( fmt, aInteger );
         }
-      void AppendInt( PRUint64 aInteger )
+      void AppendInt( uint64_t aInteger )
                  { AppendPrintf( "%llu", aInteger ); }
-      void AppendInt( PRUint64 aInteger, int aRadix )
+      void AppendInt( uint64_t aInteger, int aRadix )
         {
           const char *fmt = aRadix == 10 ? "%llu" : aRadix == 8 ? "%llo" : "%llx";
           AppendPrintf( fmt, aInteger );
@@ -439,39 +477,44 @@ class nsTSubstring_CharT
       /**
        * Append the given float to this string 
        */
-      void AppendFloat( float aFloat )
-                      { DoAppendFloat(aFloat, 6); }
-      void AppendFloat( double aFloat )
-                      { DoAppendFloat(aFloat, 15); }
-  private:
-      void NS_FASTCALL DoAppendFloat( double aFloat, int digits );
+      void NS_FASTCALL AppendFloat( float aFloat );
+      void NS_FASTCALL AppendFloat( double aFloat );
   public:
 
     // AppendLiteral must ONLY be applied to an actual literal string.
     // Do not attempt to use it with a regular char* pointer, or with a char
-    // array variable. Use AppendASCII for those.
-#ifdef NS_DISABLE_LITERAL_TEMPLATE
-      void AppendLiteral( const char* str )
-                  { AppendASCII(str); }
-#else
+    // array variable. Use Append or AppendASCII for those.
+      template<int N>
+      void AppendLiteral( const char_type (&str)[N] )                                             { ReplaceLiteral(mLength, 0, str, N - 1); }
+#ifdef CharT_is_PRUnichar
       template<int N>
       void AppendLiteral( const char (&str)[N] )
                   { AppendASCII(str, N-1); }
-      template<int N>
-      void AppendLiteral( char (&str)[N] )
-                  { AppendASCII(str, N-1); }
 #endif
 
       self_type& operator+=( char_type c )                                                       { Append(c);        return *this; }
       self_type& operator+=( const char_type* data )                                             { Append(data);     return *this; }
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      self_type& operator+=( char16ptr_t data )                                                  { Append(data);     return *this; }
+#endif
       self_type& operator+=( const self_type& str )                                              { Append(str);      return *this; }
       self_type& operator+=( const substring_tuple_type& tuple )                                 { Append(tuple);    return *this; }
 
       void Insert( char_type c, index_type pos )                                                 { Replace(pos, 0, c); }
       void Insert( const char_type* data, index_type pos, size_type length = size_type(-1) )     { Replace(pos, 0, data, length); }
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      void Insert( char16ptr_t data, index_type pos, size_type length = size_type(-1) )
+        { Insert(static_cast<const char16_t*>(data), pos, length); }
+#endif
       void Insert( const self_type& str, index_type pos )                                        { Replace(pos, 0, str); }
       void Insert( const substring_tuple_type& tuple, index_type pos )                           { Replace(pos, 0, tuple); }
 
+    // InsertLiteral must ONLY be applied to an actual literal string.
+    // Do not attempt to use it with a regular char* pointer, or with a char
+    // array variable. Use Insert for those.
+      template<int N>
+      void InsertLiteral( const char_type (&str)[N], index_type pos )                            { ReplaceLiteral(pos, 0, str, N - 1); }
+
       void Cut( index_type cutStart, size_type cutLength )                                       { Replace(cutStart, cutLength, char_traits::sEmptyBuffer, 0); }
 
 
@@ -480,8 +523,11 @@ class nsTSubstring_CharT
          */
 
         /**
-         * Attempts to set the capacity to the given size, without affecting
-         * the length of the string. Also ensures that the buffer is mutable.
+         * Attempts to set the capacity to the given size in number of 
+         * characters, without affecting the length of the string.
+         * There is no need to include room for the null terminator: it is
+         * the job of the string class.
+         * Also ensures that the buffer is mutable.
          */
       void NS_FASTCALL SetCapacity( size_type newCapacity );
       bool NS_FASTCALL SetCapacity( size_type newCapacity, const fallible_t& ) NS_WARN_UNUSED_RESULT;
@@ -512,7 +558,7 @@ class nsTSubstring_CharT
           *data = mData;
           return mLength;
         }
-        
+
         /**
          * Get a pointer to the string's internal buffer, optionally resizing
          * the buffer first.  If size_type(-1) is passed for newLen, then the
@@ -526,7 +572,7 @@ class nsTSubstring_CharT
       size_type GetMutableData( char_type** data, size_type newLen = size_type(-1) )
         {
           if (!EnsureMutable(newLen))
-            NS_RUNTIMEABORT("OOM");
+            NS_ABORT_OOM(newLen == size_type(-1) ? mLength : newLen);
 
           *data = mData;
           return mLength;
@@ -536,7 +582,7 @@ class nsTSubstring_CharT
         {
           if (!EnsureMutable(newLen))
             {
-              *data = nsnull;
+              *data = nullptr;
               return 0;
             }
 
@@ -544,6 +590,18 @@ class nsTSubstring_CharT
           return mLength;
         }
 
+#if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
+      size_type GetMutableData( wchar_t** data, size_type newLen = size_type(-1) )
+        {
+          return GetMutableData(reinterpret_cast<char16_t**>(data), newLen);
+        }
+
+      size_type GetMutableData( wchar_t** data, size_type newLen, const fallible_t& )
+        {
+    return GetMutableData(reinterpret_cast<char16_t**>(data), newLen, fallible_t());
+        }
+#endif
+
 
         /**
          * string data is never null, but can be marked void.  if true, the
@@ -560,7 +618,7 @@ class nsTSubstring_CharT
          *  @param  aOffset -- where in this string to start stripping chars
          */
          
-      void StripChar( char_type aChar, PRInt32 aOffset=0 );
+      void StripChar( char_type aChar, int32_t aOffset=0 );
 
         /**
          *  This method is used to remove all occurrences of aChars from this
@@ -570,7 +628,7 @@ class nsTSubstring_CharT
          *  @param  aOffset -- where in this string to start stripping chars
          */
 
-      void StripChars( const char_type* aChars, PRUint32 aOffset=0 );
+      void StripChars( const char_type* aChars, uint32_t aOffset=0 );
 
         /**
          * If the string uses a shared buffer, this method
@@ -593,7 +651,7 @@ class nsTSubstring_CharT
          * base type, which helps avoid converting to nsTAString.
          */
       nsTSubstring_CharT(const substring_tuple_type& tuple)
-        : mData(nsnull),
+        : mData(nullptr),
           mLength(0),
           mFlags(F_NONE)
         {
@@ -609,23 +667,34 @@ class nsTSubstring_CharT
         // XXXbz or can I just include nscore.h and use NS_BUILD_REFCNT_LOGGING?
 #if defined(DEBUG) || defined(FORCE_BUILD_REFCNT_LOGGING)
 #define XPCOM_STRING_CONSTRUCTOR_OUT_OF_LINE
-      nsTSubstring_CharT( char_type *data, size_type length, PRUint32 flags );
+      nsTSubstring_CharT( char_type *data, size_type length, uint32_t flags );
 #else
 #undef XPCOM_STRING_CONSTRUCTOR_OUT_OF_LINE
-      nsTSubstring_CharT( char_type *data, size_type length, PRUint32 flags )
+      nsTSubstring_CharT( char_type *data, size_type length, uint32_t flags )
         : mData(data),
           mLength(length),
           mFlags(flags) {}
 #endif /* DEBUG || FORCE_BUILD_REFCNT_LOGGING */
 
-      size_t SizeOfExcludingThisMustBeUnshared(nsMallocSizeOfFun mallocSizeOf)
+      size_t SizeOfExcludingThisMustBeUnshared(mozilla::MallocSizeOf mallocSizeOf)
         const;
-      size_t SizeOfIncludingThisMustBeUnshared(nsMallocSizeOfFun mallocSizeOf)
+      size_t SizeOfIncludingThisMustBeUnshared(mozilla::MallocSizeOf mallocSizeOf)
         const;
 
-      size_t SizeOfExcludingThisIfUnshared(nsMallocSizeOfFun mallocSizeOf)
+      size_t SizeOfExcludingThisIfUnshared(mozilla::MallocSizeOf mallocSizeOf)
         const;
-      size_t SizeOfIncludingThisIfUnshared(nsMallocSizeOfFun mallocSizeOf)
+      size_t SizeOfIncludingThisIfUnshared(mozilla::MallocSizeOf mallocSizeOf)
+        const;
+
+        /**
+         * WARNING: Only use these functions if you really know what you are
+         * doing, because they can easily lead to double-counting strings.  If
+         * you do use them, please explain clearly in a comment why it's safe
+         * and won't lead to double-counting.
+         */
+      size_t SizeOfExcludingThisEvenIfShared(mozilla::MallocSizeOf mallocSizeOf)
+        const;
+      size_t SizeOfIncludingThisEvenIfShared(mozilla::MallocSizeOf mallocSizeOf)
         const;
 
     protected:
@@ -638,7 +707,7 @@ class nsTSubstring_CharT
 
       char_type*  mData;
       size_type   mLength;
-      PRUint32    mFlags;
+      uint32_t    mFlags;
 
         // default initialization 
       nsTSubstring_CharT()
@@ -648,7 +717,7 @@ class nsTSubstring_CharT
 
         // version of constructor that leaves mData and mLength uninitialized
       explicit
-      nsTSubstring_CharT( PRUint32 flags )
+      nsTSubstring_CharT( uint32_t flags )
         : mFlags(flags) {}
 
         // copy-constructor, constructs as dependent on given object
@@ -683,7 +752,7 @@ class nsTSubstring_CharT
          *
          * XXX we should expose a way for subclasses to free old_data.
          */
-      bool NS_FASTCALL MutatePrep( size_type capacity, char_type** old_data, PRUint32* old_flags );
+      bool NS_FASTCALL MutatePrep( size_type capacity, char_type** old_data, uint32_t* old_flags );
 
         /**
          * this function prepares a section of mData to be modified.  if
@@ -708,8 +777,8 @@ class nsTSubstring_CharT
       bool ReplacePrep(index_type cutStart, size_type cutLength,
                        size_type newLength) NS_WARN_UNUSED_RESULT
       {
-        cutLength = NS_MIN(cutLength, mLength - cutStart);
-        PRUint32 newTotalLen = mLength - cutLength + newLength;
+        cutLength = XPCOM_MIN(cutLength, mLength - cutStart);
+        uint32_t newTotalLen = mLength - cutLength + newLength;
         if (cutStart == mLength && Capacity() > newTotalLen) {
           mFlags &= ~F_VOIDED;
           mData[newTotalLen] = char_type(0);
@@ -759,14 +828,16 @@ class nsTSubstring_CharT
         /**
          * this helper function stores the specified dataFlags in mFlags
          */
-      void SetDataFlags(PRUint32 dataFlags)
+      void SetDataFlags(uint32_t dataFlags)
         {
           NS_ASSERTION((dataFlags & 0xFFFF0000) == 0, "bad flags");
           mFlags = dataFlags | (mFlags & 0xFFFF0000);
         }
 
-      static PRIntn AppendFunc( void* arg, const char* s, PRUint32 len);
-      void AppendPrintf( const char* format, va_list ap );
+      void NS_FASTCALL AssignLiteral( const char_type* data, size_type length );
+      void NS_FASTCALL ReplaceLiteral( index_type cutStart, size_type cutLength, const char_type* data, size_type length );
+
+      static int AppendFunc( void* arg, const char* s, uint32_t len);
 
     public:
 
@@ -786,6 +857,7 @@ class nsTSubstring_CharT
           F_SHARED       = 1 << 2,  // mData points to a heap-allocated, shared buffer
           F_OWNED        = 1 << 3,  // mData points to a heap-allocated, raw buffer
           F_FIXED        = 1 << 4,  // mData points to a fixed-size writable, dependent buffer
+          F_LITERAL      = 1 << 5,  // mData points to a string literal; F_TERMINATED will also be set
 
           // class flags are in the upper 16-bits
           F_CLASS_FIXED  = 1 << 16   // indicates that |this| is of type nsTFixedString
@@ -853,6 +925,13 @@ bool operator==( const nsTSubstring_CharT::base_string_type& lhs, const nsTSubst
     return lhs.Equals(rhs);
   }
 
+inline
+bool operator==( const nsTSubstring_CharT::base_string_type& lhs, const nsTSubstring_CharT::char_type* rhs )
+  {
+    return lhs.Equals(rhs);
+  }
+
+
 inline
 bool operator>=( const nsTSubstring_CharT::base_string_type& lhs, const nsTSubstring_CharT::base_string_type& rhs )
   {
